require 'sketchup.rb'
require 'parametric.rb'
#####################################################################################################
#    ARC TOOL
###################################################################################################
# Copyright 2009, Didier Bur
## Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#----------------------------------------------------------------------------
# Name : Arc Tool
# Description : ߵĹߣ12ԲĻߣ23㻭
# Menu Item : Draw / Arc: Center and Point
# Date : 31 Oct 2007
# Type : Tool
#----------------------------------------------------------------------------

module Arc3pts
class  Arc3ptsTool
def initialize
    @ip1 = nil
    @ip2 = nil
    @ip3 = nil
    @xdown = 0
    @ydown = 0
    $arc_points=[]
    @state = 0
    #default values
    $arc_prec = 20 if not $arc_prec
end
# When the tool is first selected.
def activate
    Sketchup.active_model.start_operation "Arc: Center, Point"
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip3 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    self.reset(nil)
end
# When the tool is deactivated because a different tool was selected
def deactivate(view)
    view.invalidate if @drawn
end
# Whenever the user moves the mouse.
def onMouseMove(flags, x, y, view)
#puts "state " + @state.to_s
    #if( @state == 0 )
        # We are getting the first point (start origin)
        @ip.pick view, x, y
        if( @ip != @ip1 )
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
            # set the tooltip that should be displayed to this point
            view.tooltip = @ip1.tooltip
        end
    #end
    
  case @state
    when 0
      view.tooltip = "First point of arc"
    when 1
      view.tooltip = "Second point of arc"
    when 2
      view.tooltip = "Third point of arc"
  end
end
# When the user presses the left mouse button.
def onLButtonDown(flags, x, y, view)
    # When the user clicks the first time, we switch to getting the
    # second point.  When they click a second time we create the line
    
    #puts "State: " + @state.to_s
    @ip1.pick view, x, y
#if( @ip1.valid? )
  case @state
    when 0
      Sketchup::set_status_text("First point of arc", SB_PROMPT)
      @xdown = x
      @ydown = y
      view.tooltip = @ip1.tooltip if( @ip1.valid? )
      $arc_points.push @ip1.position
      @state = 1
      view.invalidate
    when 1
      Sketchup::set_status_text("Second point of arc", SB_PROMPT)
      @xdown = x
      @ydown = y
      @ip2.pick view, x, y, @ip1
      view.tooltip = @ip2.tooltip if( @ip2.valid? )
      $arc_points.push @ip2.position
      @state = 2
      view.invalidate 
    when 2
      Sketchup::set_status_text("Third Point of arc", SB_PROMPT)
      @xdown = x
      @ydown = y
      @ip3.pick view, x, y, @ip1
      view.tooltip = @ip3.tooltip if( @ip3.valid? )
      $arc_points.push @ip3.position
      @state = 3
      view.invalidate
    # Clear any inference lock
    view.lock_inference
  end #case
#end
end
# user releases the left mouse button.
def onLButtonUp(flags, x, y, view)
  self.reset(view)
  #if @state == 3
    #create temp circle
    #$arc_help_circle=Sketchup.active_model.entities.add_circle($arc_points[0], @vz, $arc_rad, 55)
  #end
  self.draw_geometry(view) if @state == 3
end
# User presses a key on the keyboard.
# We are checking it here to see if the user pressed the shift key
# so that we can do inference locking
def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
        # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2
        elsif( @state == 2 && @ip3.valid? )
            view.lock_inference @ip3
        end
    end
end
# User releases the key
# We use this to unlock the interence
# If the user holds down the shift key for more than 1/2 second, then we
# unlock the inference on the release.  Otherwise, the user presses shift
# once to lock and a second time to unlock.
def onKeyUp(key, repeat, flags, view)
  return if not @state == 4
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end
# onUserText is called when the user enters something into the VCB
def onUserText(text, view)
    return if not @state == 4
end
# The draw method is called whenever the view is refreshed.  It lets the
# tool draw any temporary geometry that it needs to.
def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
    end
    
  if $arc_points.length == 1
    view.draw_points $arc_points, 8, 1, "black"
  end
  
  if $arc_points.length == 2
    view.line_width = 2
    view.draw_points $arc_points[0], 8, 1, "black"
    view.draw_points $arc_points[1], 8, 1, "black"
    @v1 = $arc_points[1]-$arc_points[0]
  end
  
  if $arc_points.length == 3
    view.line_width = 2
    view.draw_points $arc_points[0], 8, 1, "black"
    view.draw_points $arc_points[1], 8, 1, "black"
    view.draw_points $arc_points[2], 8, 1, "black"
    @v2=$arc_points[2]-$arc_points[1]
    @vz=($arc_points[0] - $arc_points[1]).cross($arc_points[0] - $arc_points[2])
  end
  view.lock_inference
end
# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
    self.draw_geometry(view) if @state == 3
    if $arc_points.length < 3
      # We got less than 3 points
      UI.messagebox($arc_points.length.to_s + " points are not enough to draw an arc!")
      self.reset(view)
    end
end
# Reset the tool back to its initial state
def reset(view)
    # This variable keeps track of which point we are currently getting
    #@state = 0
    case @state
      when 0
        Sketchup::set_status_text("First point", SB_PROMPT)
      when 1
        Sketchup::set_status_text("Second point", SB_PROMPT)
      when 2
        Sketchup::set_status_text("Third point", SB_PROMPT)
        #Sketchup::set_status_text("", SB_PROMPT)
    end
    
    # clear the InputPoint
    @ip1.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    @drawn = false
    @dragging = false
end
def draw_geometry(view)
  pts = []
  prompt = [": "]
  params = [$arc_prec]
  result = inputbox(prompt, params, "Arc Parameter")
  if result
    $arc_prec = result[0]
  else
    return
  end
  entities = Sketchup.active_model.entities
#compute center of arc
@v1m=@v1.clone
@v1m.length = @v1.length/2.0
t=Geom::Transformation.translation @v1m
@mid_v1= $arc_points[0].transform t
#entities.add_cpoint @mid_v1
r1 = Geom::Transformation.rotation(@mid_v1, @vz, Math::PI/2.0)
@v1mr=@v1m.transform r1
@v2m=@v2.clone
@v2m.length = @v2.length/2.0
t=Geom::Transformation.translation @v2m
@mid_v2= $arc_points[1].transform t
#entities.add_cpoint @mid_v2
#entities.add_line($arc_points[0], $arc_points[1])
#entities.add_line($arc_points[1], $arc_points[2])
r2 = Geom::Transformation.rotation(@mid_v2, @vz, Math::PI/2.0)
@v2mr=@v2m.transform r2
@center=Geom.intersect_line_line([@mid_v1, @v1mr], [@mid_v2, @v2mr])
#entities.add_cpoint @center
#draw arc
arc_angle = ($arc_points[2] - @center).angle_between(($arc_points[0] - @center))
@arc_rad = @center.distance $arc_points[0]
if arc_angle > Math::PI
  arc_angle = 0 - arc_angle
end
entities.add_arc @center, ($arc_points[0] - @center), @vz, @arc_rad, 0, arc_angle, $arc_prec
# reset points array and exit tool
$arc_points=[]
Sketchup.send_action "selectSelectionTool:"
Sketchup.active_model.commit_operation
end
end # class
#---------------------------------------
# Select the Arccenter tool               
#---------------------------------------
def Arc3pts.tool
    Sketchup.active_model.select_tool Arc3ptsTool.new
end
end # module Arccenter
module Arccenter
class  ArccenterTool
def initialize
    @ip1 = nil
    @ip2 = nil
    @ip3 = nil
    @xdown = 0
    @ydown = 0
    $arc_points=[]
    @state = 0
    #default values
    $arc_prec = 20 if not $arc_prec
end
# When the tool is first selected.
def activate
    Sketchup.active_model.start_operation "Arc: Center & 2 points"
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip3 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    self.reset(nil)
end
# When the tool is deactivated because a different tool was selected
def deactivate(view)
    view.invalidate if @drawn
end
# Whenever the user moves the mouse.
def onMouseMove(flags, x, y, view)
#puts "state " + @state.to_s
    #if( @state == 0 )
        # We are getting the first point (start origin)
        @ip.pick view, x, y
        if( @ip != @ip1 )
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
            # set the tooltip that should be displayed to this point
            view.tooltip = @ip1.tooltip
        end
    #end
    
  case @state
    when 0
      view.tooltip = "Center point"
    when 1
      view.tooltip = "Start point"
    when 2
      view.tooltip = "End point"
  end
  
end
# When the user presses the left mouse button.
def onLButtonDown(flags, x, y, view)
    # When the user clicks the first time, we switch to getting the
    # second point.  When they click a second time we create the line
    
    #puts "State: " + @state.to_s
    @ip1.pick view, x, y
#if( @ip1.valid? )
  case @state
    when 0
      Sketchup::set_status_text("Center point of arc", SB_PROMPT)
      @xdown = x
      @ydown = y
      view.tooltip = @ip1.tooltip if( @ip1.valid? )
      $arc_points.push @ip1.position
      @state = 1
      view.invalidate
    when 1
      Sketchup::set_status_text("Start point of arc", SB_PROMPT)
      @xdown = x
      @ydown = y
      @ip2.pick view, x, y, @ip1
      view.tooltip = @ip2.tooltip if( @ip2.valid? )
      $arc_points.push @ip2.position
      @state = 2
      view.invalidate 
    when 2
      Sketchup::set_status_text("End point of arc", SB_PROMPT)
      @xdown = x
      @ydown = y
      @ip3.pick view, x, y, @ip1
      view.tooltip = @ip3.tooltip if( @ip3.valid? )
      $arc_points.push @ip3.position
      @state = 3
      view.invalidate
    # Clear any inference lock
    view.lock_inference
  end #case
#end
end
# user releases the left mouse button.
def onLButtonUp(flags, x, y, view)
  self.reset(view)
  #if @state == 3
    #create temp circle
    #$arc_help_circle=Sketchup.active_model.entities.add_circle($arc_points[0], @vz, $arc_rad, 55)
  #end
  self.draw_geometry(view) if @state == 3
end
# User presses a key on the keyboard.
# We are checking it here to see if the user pressed the shift key
# so that we can do inference locking
def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
        # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2
        elsif( @state == 2 && @ip3.valid? )
            view.lock_inference @ip3
        elsif( @state == 3 && @ip4.valid? )
            view.lock_inference @ip4
        end
    end
end
# User releases the key
# We use this to unlock the interence
# If the user holds down the shift key for more than 1/2 second, then we
# unlock the inference on the release.  Otherwise, the user presses shift
# once to lock and a second time to unlock.
def onKeyUp(key, repeat, flags, view)
  return if not @state == 4
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end
# onUserText is called when the user enters something into the VCB
def onUserText(text, view)
    return if not @state == 4
end
# The draw method is called whenever the view is refreshed.  It lets the
# tool draw any temporary geometry that it needs to.
def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
    end
    
  if $arc_points.length == 1
    view.draw_points $arc_points, 8, 4, "black"
  end
  
  if $arc_points.length == 2
    view.draw_points $arc_points[0], 8, 4, "black"
    view.draw_points $arc_points[1], 8, 1, "red"
    $arc_rad=$arc_points[0].distance $arc_points[1]
  end
  
  if $arc_points.length == 3
    view.draw_points $arc_points[0], 8, 4, "black"
    view.draw_points $arc_points[1], 8, 1, "red"
    view.draw_points $arc_points[2], 8, 1, "red"
    #compute middle point
    v=$arc_points[1]-$arc_points[0]
    v.length = v.length/3.0
  end
  view.lock_inference
end
# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
    self.draw_geometry(view) if @state == 3
    if $arc_points.length < 3
      # We got less than 3 points
      UI.messagebox($arc_points.length.to_s + " points are not enough to draw an arc!")
      self.reset(view)
    end
end
# Reset the tool back to its initial state
def reset(view)
    # This variable keeps track of which point we are currently getting
    #@state = 0
    case @state
      when 0
        Sketchup::set_status_text("Center point", SB_PROMPT)
      when 1
        Sketchup::set_status_text("Start point", SB_PROMPT)
      when 2
        Sketchup::set_status_text("End point", SB_PROMPT)
        #Sketchup::set_status_text("", SB_PROMPT)
    end
    
    # clear the InputPoint
    @ip1.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    @drawn = false
    @dragging = false
end
def draw_geometry(view)
  pts = []
  prompt = [": "]
  params = [$arc_prec]
  result = inputbox(prompt, params, "Arc Parameter")
  if result
    $arc_prec = result[0]
  else
    return
  end
  entities = Sketchup.active_model.entities
#draw arc
arc_angle = ($arc_points[1] - $arc_points[0]).angle_between(($arc_points[2] - $arc_points[0]))
if arc_angle > Math::PI
  arc_angle = 0 - arc_angle
end
@vz=($arc_points[0] - $arc_points[1]).cross($arc_points[0] - $arc_points[2])
entities.add_arc $arc_points[0], ($arc_points[1] - $arc_points[0]), @vz, $arc_rad, 0, arc_angle, $arc_prec
# reset points array and exit tool
$arc_points=[]
Sketchup.send_action "selectSelectionTool:"
Sketchup.active_model.commit_operation
end
end # class
#---------------------------------------
# Select the Arccenter tool               
#---------------------------------------
def Arccenter.tool
    Sketchup.active_model.select_tool ArccenterTool.new
end
end # module Arccenter
#-----------------------------------------------------------------------------
# Shortcut for selecting the new tool within the UI                           
#-----------------------------------------------------------------------------
#if not $arc3pts_loaded
#  main_menu = UI.menu("Draw").add_item("Arc: 3 Points") { Arc3pts.tool }
#  main_menu = UI.menu("Draw").add_item("Arc: Center and 2 points") { Arccenter.tool }
#end
#$arc3pts_loaded = true
###################################################################################################
####   ELLIPSE
###################################################################################################
# Copyright 2007, Didier Bur
#
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#----------------------------------------------------------------------------
# Name : Ellipse Tool
# Description : ԲĹ.
# Menu Item : Draw / Ellipse
# Usage : 1. Click 2 points for major axis
#         2. Click a point to orient the ellipse plane
#         3. Click a point for half minor axis
# Date : 29 Sept 2007
# Type : Tool
#----------------------------------------------------------------------------

module Ellipse
class  EllipseTool
def initialize
    @ip1 = nil
    @ip2 = nil
    @ip3 = nil
    @xdown = 0
    @ydown = 0
    $axis_points=[]
    @state = 0
    #default values
    $ell_type = "Curve" if not $ell_type
    $ell_cf = "Yes" if not $ell_cf
    $ell_prec = 20 if not $ell_prec
    $ell_kg = "No" if not $ell_kg
end
# When the tool is first selected.
def activate
    Sketchup.active_model.start_operation "Ellipse"
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip3 = Sketchup::InputPoint.new
    @ip4 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    self.reset(nil)
end
# When the tool is deactivated because a different tool was selected
def deactivate(view)
    view.invalidate if @drawn
end
# Whenever the user moves the mouse.
def onMouseMove(flags, x, y, view)
#puts "state " + @state.to_s
    #if( @state == 0 )
        # We are getting the first point (start origin)
        @ip.pick view, x, y
        if( @ip != @ip1 )
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
            # set the tooltip that should be displayed to this point
            view.tooltip = @ip1.tooltip
        end
    #end
    
  case @state
    when 0
      view.tooltip = "First major axis point"
    when 1
      view.tooltip = "Second major axis point"
    when 2
      view.tooltip = "Plane point"
    when 3
      view.tooltip = "Minor axis point"
  end
  
end
# When the user presses the left mouse button.
def onLButtonDown(flags, x, y, view)
    # When the user clicks the first time, we switch to getting the
    # second point.  When they click a second time we create the line
    
    #puts "State: " + @state.to_s
    @ip1.pick view, x, y
#if( @ip1.valid? )
  case @state
    when 0
      Sketchup::set_status_text("First point of major axis", SB_PROMPT)
      @xdown = x
      @ydown = y
      view.tooltip = @ip1.tooltip if( @ip1.valid? )
      $axis_points.push @ip1.position
      @state = 1
      view.invalidate
    when 1
      Sketchup::set_status_text("Second point of major axis", SB_PROMPT)
      @xdown = x
      @ydown = y
      @ip2.pick view, x, y, @ip1
      view.tooltip = @ip2.tooltip if( @ip2.valid? )
      $axis_points.push @ip2.position
      @state = 2
      view.invalidate 
    when 2
      Sketchup::set_status_text("Point of ellipse plane", SB_PROMPT)
      @xdown = x
      @ydown = y
      @ip3.pick view, x, y, @ip1
      view.tooltip = @ip3.tooltip if( @ip3.valid? )
      $axis_points.push @ip3.position
      @state = 3
      view.invalidate
    when 3
      Sketchup::set_status_text("Point of minor axis", SB_PROMPT)
      @xdown = x
      @ydown = y
      @ip4.pick view, x, y, @ip1
      view.tooltip = @ip4.tooltip if( @ip4.valid? )
      $axis_points.push @ip4.position
      @state = 4
      view.invalidate
    # Clear any inference lock
    view.lock_inference
  end #case
#end
end
# user releases the left mouse button.
def onLButtonUp(flags, x, y, view)
  self.reset(view)
  if @state == 3
    #create temp help axis
    hl1=@midp0.offset(@vy)
    hl2=@midp0.offset(@vy.reverse)
    $ell_hl=Sketchup.active_model.entities.add_cline(hl1,hl2)
  end
  self.draw_geometry(view) if @state == 4
end
# User presses a key on the keyboard.
# We are checking it here to see if the user pressed the shift key
# so that we can do inference locking
def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
        # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2
        elsif( @state == 2 && @ip3.valid? )
            view.lock_inference @ip3
        elsif( @state == 3 && @ip4.valid? )
            view.lock_inference @ip4
        end
    end
end
# User releases the key
# We use this to unlock the interence
# If the user holds down the shift key for more than 1/2 second, then we
# unlock the inference on the release.  Otherwise, the user presses shift
# once to lock and a second time to unlock.
def onKeyUp(key, repeat, flags, view)
  return if not @state == 5
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end
# onUserText is called when the user enters something into the VCB
def onUserText(text, view)
    return if not @state == 5
end
# The draw method is called whenever the view is refreshed.  It lets the
# tool draw any temporary geometry that it needs to.
def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
    end
    
  if $axis_points.length == 1
    view.draw_points $axis_points, 8, 1, "black"
  end
  
  if $axis_points.length == 2
    view.line_width = 2
    view.draw_points $axis_points[0], 8, 1, "black"
    view.draw_points $axis_points[1], 8, 1, "black"
    #compute middle point
    v=$axis_points[1]-$axis_points[0]
    v.length = v.length/2.0
    t=Geom::Transformation.translation v
    @midp0 = $axis_points[0].transform t
    view.draw_points @midp0, 7, 4, "red"
    view.line_stipple = "..."
    view.draw_line $axis_points[0], $axis_points[1]
  end
  
  if $axis_points.length == 3
    view.line_width = 2
    view.draw_points $axis_points[0], 8, 1, "black"
    view.draw_points $axis_points[1], 8, 1, "black"
    view.draw_points $axis_points[2], 8, 1, "black"
    #compute middle point
    v=$axis_points[1]-$axis_points[0]
    v.length = v.length/2.0
    t=Geom::Transformation.translation v
    @midp0 = $axis_points[0].transform t
    view.draw_points @midp0, 7, 4, "red"
    view.line_stipple = "..."
    view.draw_line $axis_points[0], $axis_points[1]
    #temp axes display
    view.drawing_color = "red"
    view.line_stipple = ""
    v.length = v.length/2.0
    #view.draw_points @midp0.offset(v), 8, 1, "blue"
    x_axis_end = @midp0.offset(v)
    view.draw_line @midp0,x_axis_end
    vz=($axis_points[0] - $axis_points[1]).cross($axis_points[0] - $axis_points[2])
    vz.length=v.length
    r=Geom::Transformation.rotation @midp0,vz,(Math::PI / 2.0)
    y_axis_end=x_axis_end.transform r
    @vy=@midp0-y_axis_end
    @vy.length=v.length*10
    view.drawing_color = "green"
    view.draw_line @midp0,y_axis_end
    view.drawing_color = "blue"
    z_axis_end = @midp0.offset(vz)
    view.draw_line @midp0,z_axis_end
  end
  view.lock_inference
end
# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
    self.draw_geometry(view) if @state == 3
    if $axis_points.length < 3
      # We got less than 6 points
      UI.messagebox($axis_points.length.to_s + " points are not enough to draw ellipse!")
      self.reset(view)
    end
end
# Reset the tool back to its initial state
def reset(view)
    # This variable keeps track of which point we are currently getting
    #@state = 0
    case @state
      when 0
        Sketchup::set_status_text("һ ()", SB_PROMPT)
      when 1
        Sketchup::set_status_text("ڶ ()", SB_PROMPT)
      when 2
        Sketchup::set_status_text("ڶ ()", SB_PROMPT)
        Sketchup::set_status_text("", SB_PROMPT)
    end
    
    # clear the InputPoint
    @ip1.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    @drawn = false
    @dragging = false
end
def draw_geometry(view)
  pts = []
  list1=["Segments","Curve"].join("|")
  list2=["Yes", "No"].join("|")
  list3=["Yes", "No"].join("|")
  drops=[list1,list2, list3]
  prompt = ["Type: ","Create face: ","Keep group: ","Precision: "]
  params = [$ell_type,$ell_cf,$ell_kg, $ell_prec]
  result = inputbox(prompt, params, drops, "Ellipse Parameters")
  if result
    $ell_type = result[0]
    $ell_cf = result[1]
    $ell_prec = result[3]
    $ell_kg = result[2]
  else
    return
  end
  
  w = ($axis_points[0].distance $axis_points[1]) / 2.0
  h = @midp0.distance $axis_points[3]
  
  #x_axis=Geom::Vector3d.new 1,0,0
  #major_axis=$axis_points[0] - $axis_points[1]
  #theta=x_axis.angle_between major_axis
  
  #rotate = Geom::Transformation.rotation @midp0, (Geom::Vector3d.new[0,0,1]), theta
  #rotate = Geom::Transformation.rotation @midp0, major_axis, theta
  
  entities = Sketchup.active_model.entities
  ell_grp = entities.add_group
  ell_ents = ell_grp.entities
  #draw ellipse at origin in group
  for ang in (0..$ell_prec)
    x = w*(Math::cos ang*((Math::PI*2)/$ell_prec))
    y = h*(Math::sin ang*((Math::PI*2)/$ell_prec))
    p = Geom::Point3d.new(x,y,0)
    #p.transform! translate
    #p.transform! rotate
    pts.push p
  end
if $ell_type == "Curve"
  c = ell_ents.add_curve pts
  edges = c
  else
  edges = ell_ents.add_edges pts
end
if $ell_cf == "Yes" and $ell_type == "Curve"
  f=edges[edges.length-1].find_faces
end
if $ell_cf == "Yes" and $ell_type == "Segments"
  edges.each do |e|
    a = e.find_faces
  end
end
#translate group
#translate = Geom::Transformation.translation @midp0
#ell_grp.transform! translate
ax=$axis_points[1]-$axis_points[0]
ay=$axis_points[3]-$axis_points[0]
az=($axis_points[0] - $axis_points[1]).cross($axis_points[0] - $axis_points[3])
trans=Geom::Transformation.axes @midp0, ax, ay, az
ell_grp.transform! trans
#explode group
if $ell_kg != "Yes"
  ell_grp.explode
end
#delete temp help line
$ell_hl.erase!
# reset points array and exit tool
  $axis_points=[]
  Sketchup.send_action "selectSelectionTool:"
  Sketchup.active_model.commit_operation
end
end # class
#---------------------------------------
# Select the Ellipse tool               
#---------------------------------------
def Ellipse.tool
    Sketchup.active_model.select_tool EllipseTool.new
end
end # module Ellipse
#-----------------------------------------------------------------------------
# Shortcut for selecting the new tool within the UI                           
#-----------------------------------------------------------------------------
#if not $ellipse_loaded
#  main_menu = UI.menu("Draw").add_item("Ellipse") { Ellipse.tool }
#end
#$ellipse_loaded = true
###################################################################################################
####   
####      DOUBLE LINE
####
###################################################################################################
# Copyright 2008, D. Bur                                                     
#----------------------------------------------------------------------------
# Name        :   dline                                                      
# Description :   A tool to draw wall or walls footprints                    
# Menu Item   :   Draw -> Double line                                        
# Context Menu:   NONE                                                       
# Author      :   Didier Bur                                                 
# Date        :   19/08/2008                                                 
# Type        :   Tool                                                       
#----------------------------------------------------------------------------
$debug=false
class Array
	def offsetPoints(dist)
#		return nil if dist==0
		pi=Math::PI
		if (not ((dist.class==Fixnum || dist.class==Float || dist.class==Length) && dist!=0))
			return nil
		end
		verts=self
		pts=[]
		0.upto(verts.length-1) do |a|
			if verts[a-(verts.length-1)].class==Sketchup::Vertex
				pt2=verts[a-(verts.length-1)].position
			elsif verts[a-(verts.length-1)].class==Geom::Point3d
				pt2=verts[a-(verts.length-1)]
			else
				return nil
			end
			if verts[a-1].class==Sketchup::Vertex
				pt1=verts[a-1].position
			elsif verts[a-1].class==Geom::Point3d
				pt1=verts[a-1]
			else
				return nil
			end
			if verts[a].class==Sketchup::Vertex
				pt3=verts[a].position
			elsif verts[a].class==Geom::Point3d
				pt3=verts[a]
			else
				return nil
			end
			vec1=(pt3-pt2).normalize
			vec2=(pt3-pt1).normalize
			vec3=(vec1+vec2).normalize
			if vec3.valid?
				ang=vec1.angle_between(vec2)/2
				ang=pi/2 if vec1.parallel?(vec2)
				vec3.length=dist/Math::sin(ang)
				t=Geom::Transformation.new(vec3)
				if pts.length > 0
					#if not (vec2.parallel?(pts.last.vector_to(verts[a].position.transform(t))))
          if not (vec2.parallel?(pts.last.vector_to(verts[a].transform(t))))
						t=Geom::Transformation.new(vec3.reverse)
					end
				end
				#pts.push(verts[a].position.transform(t))
        pts.push(verts[a].transform(t))
			end
		end
		pts
	end
end
class Dline_Tool
  
def initialize
  # defaults values
  $dline_just="Center" if not $dline_just
  $dline_axis="No" if not $dline_axis
  $dline_caps="Both" if not $dline_caps
  $dline_face="Yes" if not $dline_face
  $dline_group="No" if not $dline_group
  $dline_width=0.2.m if not $dline_width
  $dline_height=2.5.m if not $dline_height
  #@module=0.3.m if not @module
  @ip=Sketchup::InputPoint.new
  @ip1=Sketchup::InputPoint.new
  @ip2=Sketchup::InputPoint.new
  self.reset(nil)
end
def reset(view)
  @pts=[]
  @left_pts=[]
  @right_pts=[]
  @left_ints=[]
  @right_ints=[]
  @left_footprint=[]
  @right_footprint=[]
  @final_footprints=[]
  @state=0
  @ip.clear
  @ip1.clear
  @ip2.clear
  @on_face=nil
  @drawn = false
  Sketchup::set_status_text "First point"
  view.lock_inference if view
end
def activate
  Sketchup.active_model.active_view.lock_inference
  self.reset(nil)
  @drawn = false
  Sketchup.active_model.start_operation " Double line"
  #@old_snap=Sketchup.active_model.options[0]["LengthSnapLength"]
  #Sketchup.active_model.options[0]["LengthSnapLength"]=@module
  Sketchup::set_status_text "", SB_VCB_VALUE
  # Dialog
  prompts=["Justification: ","Caps: ","Draw axis: ","Draw face: ","Make group: ","Width: ","Height: "]
  list_just=["Left","Center","Right"].join("|")
  list_caps=["None","Start","End","Both"].join("|")
  list_face=["Yes","No"].join("|")
  dropdowns=[list_just,list_caps,list_face,list_face,list_face]
  values=[$dline_just,$dline_caps,$dline_axis,$dline_face,$dline_group,$dline_width,$dline_height]
  results=inputbox prompts,values,dropdowns,"Double line"
  Sketchup.active_model.select_tool(nil) if (not results)
  return if not results
  
  $dline_just=results[0]
  $dline_caps=results[1]
  $dline_axis=results[2]
  $dline_face=results[3]
  $dline_group=results[4]
  $dline_width=results[5] if results[5] != 0.to_l
  $dline_height=results[6]
  
  # Face management
  #@face="No" if @caps!="Both" and @face=="Yes"
  $dline_face="No" if $dline_caps!="Both" and $dline_face=="Yes"
end
def deactivate(view)
    view.invalidate if @drawn
    #Sketchup.active_model.options[0]["LengthSnapLength"]=@old_snap
    Sketchup.active_model.commit_operation
end
def onCancel(flag, view)
  # Clean view and exit the tool
  view.invalidate if @drawn
  self.reset(view)
  Sketchup.active_model.select_tool nil
end
def onReturn(view)
  if $debug
    #@pts.each { |p| Sketchup.active_model.entities.add_cpoint p }
    #@left_pts.each { |p| Sketchup.active_model.entities.add_cpoint p }
    #puts "left " + @left_pts.length.to_s + " Axis  " + @pts.length.to_s
    #@right_pts.each { |p| Sketchup.active_model.entities.add_cpoint p }
    #puts "left ints " + @left_ints.length.to_s
    #puts @left_ints
    #puts "right ints " + @right_ints.length.to_s
    #puts @right_ints
    #@left_ints.each { |p| Sketchup.active_model.entities.add_cpoint p }
    #@right_ints.each { |p| Sketchup.active_model.entities.add_cpoint p }
    #Sketchup.active_model.entities.add_line @left_footprint
    #Sketchup.active_model.entities.add_line @right_footprint
    #@final_footprints.each do |footprint|
      #Sketchup.active_model.entities.add_line footprint if footprint.length > 0
    #end
  end
  self.calculate_boundaries("open")
  self.create_geometry("open")
  self.reset(view)
end
def onUserText(text, view)
  return if not @state == 1
  return if not @ip2.valid?
  
  if text != "c" and text != "C"
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.messagebox("Cannot convert #{text} to a Length")
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    # Compute the direction and the second point
    pt1 = @ip1.position
    vec = @ip2.position - pt1
    if( vec.length == 0.0 )
        UI..messagebox("Zero length invalid !")
        return
    end
    vec.length = value
    pt2 = pt1 + vec
    # Store point
    @pts.push(pt2)
    # This point becomes initial point
    #@ip1.copy! @ip2
    @ip1= Sketchup::InputPoint.new(pt2)
  else
    # Close option entered
    if @pts.length > 2
      # Store very first point as if it was clicked
      @pts.push(@pts[0])
      self.calculate_boundaries("close")
      self.create_geometry("close")
      self.reset(view)
    else
      UI.messagebox("Cannot close until 3 segments entered.")
    end
  end
end
def onMouseMove(flags, x, y, view)
  if( @state == 0 )
    Sketchup::set_status_text "First point:"
      # Getting the first end
      @ip.pick view, x, y
      if( @ip != @ip1 )
        # if the point has changed from the last one we got
        view.invalidate if( @ip.display? or @ip1.display? )
        @ip1.copy! @ip
        # set the tooltip that should be displayed to this point
        view.tooltip = @ip1.tooltip
      end
    else
      # Getting the next end
      Sketchup::set_status_text "Next point: [Double-click]=Close  [Enter]=Draw   [Esc]=Quit  "
      Sketchup::set_status_text "Length ", SB_VCB_LABEL
      @ip2.pick view, x, y, @ip1
      
      # Update the length displayed in the VCB
      if( @ip2.valid? )
        length = @ip1.position.distance(@ip2.position)
        Sketchup::set_status_text length.to_s, SB_VCB_VALUE
      end
      view.tooltip = @ip2.tooltip if( @ip2.valid? )
      view.invalidate
  end
end
def onLButtonDown(flags, x, y, view)
  if ( @state == 0 )
    #check first point
    if( @ip1.valid? )
      @on_face=f=@ip1.face
      # Store point
      @pts.push(@ip1.position)
      @state+=1
    end
  end
  if ( @state == 1 )
    #check next point
    if( @ip2.valid? )
      @on_face=f=@ip1.face
      # Store point
      @pts.push(@ip2.position)
      # This point becomes initial point
      @ip1.copy! @ip2
    end
  end
end
def onLButtonDoubleClick(flags, x, y, view)
  return if @state==0 or @pts.length < 3
  if ( @state == 1 )
    #check next point
    if( @ip2.valid? )    
      # Store first point as last point
      @pts.push(@pts[0])
      self.calculate_boundaries("close")
      self.create_geometry("close")
      self.reset(view)
    end
  end
end
def calculate_boundaries(closure)
  rots=self.dline_compute_rotation(@pts[0],@pts[1],@pts[2],@on_face)
  rot_left=rots[0]
  rot_right=rots[1]
  
  case $dline_just
    when "Center"
      for i in 0..@pts.length-2
        segment_vec=@pts[i].vector_to(@pts[i+1])
        left_vec=segment_vec.transform(rot_left)
        right_vec=segment_vec.transform!(rot_right)
        left_vec.length=$dline_width/2.0
        right_vec.length=$dline_width/2.0
        @left_pts.push(@pts[i].offset(left_vec),@pts[i+1].offset(left_vec))
        @right_pts.push(@pts[i].offset(right_vec),@pts[i+1].offset(right_vec))
      end
    when "Left"
      for i in 0..@pts.length-2
        segment_vec=@pts[i].vector_to(@pts[i+1])
        left_vec=segment_vec.transform(rot_left)
        left_vec.length=$dline_width
        @left_pts.push(@pts[i].offset(left_vec),@pts[i+1].offset(left_vec))
        @right_pts.push(@pts[i],@pts[i+1])
      end
    when "Right"
      for i in 0..@pts.length-2
        segment_vec=@pts[i].vector_to(@pts[i+1])
        right_vec=segment_vec.transform(rot_right)
        right_vec.length=$dline_width
        @right_pts.push(@pts[i].offset(right_vec),@pts[i+1].offset(right_vec))
        @left_pts.push(@pts[i],@pts[i+1])
      end
  end #case
  
  # calculate left intersections
  j=0
  for k in 0..@pts.length-3
    @left_ints.push(Geom.intersect_line_line([@left_pts[j],@left_pts[j+1]],[@left_pts[j+2],@left_pts[j+3]]))
    j+=2
  end 
  # calculate right intersections
  j=0
  for k in 0..@pts.length-3
    @right_ints.push(Geom.intersect_line_line([@right_pts[j],@right_pts[j+1]],[@right_pts[j+2],@right_pts[j+3]]))
    j+=2
  end
  
  
  if closure=="open"
  # open wall
  
  # Sort left footprint points
  @left_footprint.push(@left_pts[0]) #start cap end
  @left_footprint=@left_footprint+@left_ints # all left intersections
  @left_footprint.push(@left_pts.last) #end cap end
  # Sort right footprint points
  @right_footprint.push(@right_pts[0]) #start cap end
  @right_footprint=@right_footprint+@right_ints # all right intersections
  @right_footprint.push(@right_pts.last) #end cap end
  # Manage caps
  case $dline_caps
    when "Both"
      @final_footprints=[@left_footprint+@right_footprint.reverse+[@left_pts[0]]]
    when "Start"
      @final_footprints=[@right_footprint.reverse+[@left_pts[0]]+@left_footprint]
    when "End"
      @final_footprints=[@left_footprint+@right_footprint.reverse]
    when "None"
      @final_footprints=[@left_footprint,@right_footprint]
  end
  
  else
  # closed wall
  # Sort left footprint points
  # start at intersection from last segment with first segment
  last_int=Geom.intersect_line_line([@left_pts[@left_pts.length-2],@left_pts.last],[@left_pts[0],@left_pts[1]])
  @left_footprint.push(last_int)
  @left_footprint=@left_footprint+@left_ints # all other left intersections
  @left_footprint.push(last_int) #close left loop
  # Sort right footprint points
  # start at intersection from last segment with first segment
  last_int=Geom.intersect_line_line([@right_pts[@right_pts.length-2],@right_pts.last],[@right_pts[0],@right_pts[1]])
  @right_footprint.push(last_int)
  @right_footprint=@right_footprint+@right_ints # all other right intersections
  @right_footprint.push(last_int) #close right loop
  # Don't manage caps !
  @final_footprints=[@left_footprint,@right_footprint]
end # if closure
end
def create_geometry(closure)
  if $dline_group=="Yes"
    g=Sketchup.active_model.active_entities.add_group
    ents=g.entities
  else
    ents=Sketchup.active_model.active_entities
  end
  case $dline_face
    when "No"
      @final_footprints.each do |footprint|
      ents.add_line footprint if footprint.length > 0
    end
    when "Yes"
      if $dline_caps=="Both"
        @base_face=ents.add_face(@final_footprints[0])
        @base_face2=ents.add_face(@final_footprints[1]) if closure == "close"
      else
        UI.messagebox("Cannot create face: cap(s) missing.")
      end
  end
  
  if @base_face and $dline_height!=0
     @base_face.pushpull(-$dline_height,false)
     if closure == "close"
      @base_face2.pushpull(-$dline_height,false)
      
      ments=Sketchup.active_model.entities if $dline_group=="No"
      ments=g.entities if $dline_group=="Yes"
      
      for i in (ments.length-40)..(ments.length)
       if (ments[i-1].typename=="Face")
        f=ments[i-1] if (ments[i-1].normal==@base_face.normal)
       end
       
      end
      f.erase! if f and f.valid?
     end
  end
  if $dline_axis=="Yes"
    for p in 0..@pts.length-2
      cline=ents.add_cline(@pts[p],@pts[p+1])
      cline.stipple="-.-"
    end
  end
end
def draw(view)
  # Show input points when valid
  if( @ip1.valid? )
    if( @ip1.display? )
      @ip1.draw(view)
      #@drawn = true
    end  
    if( @ip2.valid? )
      @ip2.draw(view) if( @ip2.display? )
      self.draw_temp_line(@ip1.position, @ip2.position, view) if (@ip1.valid? and @ip2.valid?)
      #@drawn = true
    end
  end
  # Show the previously drawn axis
  self.draw_previous_points(view)
  @drawn = true
end
# Draw temp line between ends of partition axis
def draw_temp_line(pt1, pt2, view)
  view.drawing_color = "red"
  view.line_stipple="-.-"
  view.line_width=2
  view.draw_line(pt1, pt2)
end
def draw_previous_points(view)
  view.drawing_color="green"
  view.line_stipple=0
  view.line_width=(($dline_width.to_m*5).round)*4.to_i+1
  view.draw_polyline @pts if (@pts.length >= 2)
end
def onKeyDown(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY )
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            view.lock_inference
        elsif( @state == 1 )
            view.lock_inference @ip1, @ip2
        else
            view.lock_inference @ip1
        end
    end
end
def onKeyUp(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? )
        #(Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end
# Returns abcd coef of plane defined by 3 points
def dline_plane(p1,p2,p3)
  return nil if p1.vector_to(p2).parallel? p2.vector_to(p3)
  n=(p2.vector_to(p3)).cross((p1.vector_to(p2))).normalize
  d=p1.vector_to(p2).dot(n)
  return[n[0],n[1],n[2],d]
end
# Returns normal of plane defined by 3 points
def dline_normal(p1,p2,p3)
  return (p2.vector_to(p3)).cross((p1.vector_to(p2))).normalize
end
def dline_compute_rotation(p1,p2,p3,on_face)
  if (p2!=nil and p3!=nil)
    normal=dline_normal(p1,p2,p3)
    r_left=Geom::Transformation.rotation(p1,normal,Math::PI/2.0)
    r_right=Geom::Transformation.rotation(p1,normal,-Math::PI/2.0)
  end
  if p3==nil and on_face
    normal=on_face.normal.normalize
    r_left=Geom::Transformation.rotation(p1,normal,Math::PI/2.0)
    r_right=Geom::Transformation.rotation(p1,normal,-Math::PI/2.0)
  end
  if p3==nil and not on_face
    r_left=Geom::Transformation.rotation(p1,[0,0,1],Math::PI/2.0)
    r_right=Geom::Transformation.rotation(p1,[0,0,1],-Math::PI/2.0)
  end
  return [r_left,r_right]
end
end # class Dline_Tool
# Command
def dline
  Sketchup.active_model.select_tool Dline_Tool.new
end
#Menu
#if( not file_loaded?("dline.rb") )
#  UI.menu("Draw").add_item("Double line") { dline }
#end
#file_loaded("dline.rb")
    
###################################################################################################
####   
####     HELIX
####
###################################################################################################
# DrawHelix13.rb  Peter Brown 4/8/2004
# I've now opted to load it into 'Draw' as you use it to 'draw'
# a helix and the 'draw' menu is only short.  Change it if you want.
# For a parallel helix make the end and start radii the same.
# version 12
# In this version the points are put into an array to create a curve
# rather than lines.
# version 13
# code tidied up a bit

def drawhelix
	prompts = ["End Radius","Start Radius", "Pitch", "No of Rotations", "Sections per Rotation "]
	values = [1000.mm, 1000.mm, 2000.mm, 1, 24]
	results = inputbox prompts, values, "Helix Dimensions"
	return if not results
	eradius, sradius, pitch, rotations, sections = results
 	totalsec = sections * rotations
 	angle = 2*3.14159/sections
	cosangle = Math.cos(angle)
	sinangle = Math.sin(angle)
	section = 1
	z0 = pitch / sections
	
	r1 = sradius
	dr = (eradius - sradius) / totalsec
	pts = []
	x1 = r1
	y1 = 0
	z1 = 0
	pts[pts.length] = [x1,y1,z1]
	while section < (totalsec + 1)
		x2 = (r1 + (dr * section)) * Math.cos(section * angle)
		y2 = (r1 + (dr * section)) * Math.sin(section * angle)
		z2 = section * z0
		pts[pts.length] = [x2,y2,z2]
		section += 1
	end
	model = Sketchup.active_model
	entities = model.active_entities
	group = entities.add_group
	entities = group.entities
	model.start_operation "DrawHelix"	
	entities.add_curve(pts)
	model.commit_operation
end
###################################################################################################
####   
####                             SHAPES
####
###################################################################################################
# Copyright 2004, @Last Software, Inc.
# This software is provided as an example of using the Ruby interface
# to SketchUp.
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name        :   Shapes 1.0
# Description :   Classes for creating and editing parametric shapes
# Menu Item   :   Draw->Shapes->Box
#             :   Draw->Shapes->Cylinder
#             :   Draw->Shapes->Cone
#             :   Draw->Shapes->Torus
#             :   Draw->Shapes->Tube
#             :   Draw->Shapes->Prism
#             :   Draw->Shapes->Pyramid
#             :   Draw->Shapes->Dome
# Context Menu:   Edit Box|Cylinder|Cone|Torus|Tube|Prism|Pyramid|Dome
# Usage       :   Select desired shape and fill in the dialog box that opens.
# Date        :   9/14/2004
# Type        :   Dialog Box
#-----------------------------------------------------------------------------

require 'parametric.rb'
require 'mesh_additions.rb'
#=============================================================================
# Function for generating points on a circle
def points_on_circle(center, normal, radius, numseg)
    # Get the x and y axes
    axes = Geom::Vector3d.new(normal).axes
    center = Geom::Point3d.new(center)
    xaxis = axes[0]
    yaxis = axes[1]
    
    xaxis.length = radius
    yaxis.length = radius
    # compute the points
    da = (Math::PI * 2) / numseg
    pts = []
    for i in 0...numseg do
        angle = i * da
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        vec = Geom::Vector3d.linear_combination(cosa, xaxis, sina, yaxis)
        pts.push(center + vec)
    end
    
    # close the circle
    pts.push(pts[0].clone)
    pts
end
#=============================================================================
class Box < Parametric
def create_entities(data, container)
    w = data["w"].to_l
    d = data["d"].to_l
    h = data["h"].to_l
    
    pts = [[0,0,0], [w,0,0], [w,d,0], [0,d,0], [0,0,0]]
    base = container.add_face pts
    h = -h if base.normal.dot(Z_AXIS) < 0.0
    base.pushpull h
end
def default_parameters
    defaults = {"w", 4.feet, "d", 4.feet, "h", 4.feet}
    defaults
end
def translate_key(key)
    prompt = key
    case( key )
        when "w"
            prompt = "Width"
        when "h"
            prompt = "Height"
        when "d"
            prompt = "Depth"
    end
    prompt
end
end
#=============================================================================
class Cylinder < Parametric
def create_entities(data, container)
    r = data["r"].to_l
    h = data["h"].to_l
    n = 24
    circle = container.add_circle ORIGIN, Z_AXIS, r, n
    base = container.add_face circle
    h = -h if base.normal.dot(Z_AXIS) < 0.0
    base.pushpull h
    
end
def default_parameters
    defaults = {"r", 2.feet, "h", 4.feet}
    defaults
end
def translate_key(key)
    prompt = key
    case( key )
        when "r"
            prompt = "Radius"
        when "h"
            prompt = "Height"
    end
    prompt
end
end
#=============================================================================
class Prism < Parametric
def create_entities(data, container)
    r = data["r"].to_l
    h = data["h"].to_l
    n = data["n"]
    circle = container.add_ngon ORIGIN, Z_AXIS, r, n
    base = container.add_face circle
    h = -h if base.normal.dot(Z_AXIS) < 0.0
    base.pushpull h
    
end
def default_parameters
    defaults = {"r", 2.feet, "h", 4.feet, "n", 6}
    defaults
end
def translate_key(key)
    prompt = key
    case( key )
        when "r"
            prompt = "Radius"
        when "h"
            prompt = "Height"
        when "n"
            prompt = "Number of Sides"
    end
    prompt
end
def validate_parameters(data)
    ok = true
    # make sure that there are at least 3 sides
        if( data["n"] < 3 )
        UI.messagebox "At least 3 sides required"
        ok = false
    end
    ok
end
end
#=============================================================================
class Cone < Parametric
def create_entities(data, container)
    r = data["r"].to_l
    h = data["h"].to_l
    n = 24
    # Create the base
    circle = container.add_circle ORIGIN, Z_AXIS, r, n
    base = container.add_face circle
    base_edges = base.edges 
    
    # Create the sides
    apex = [0,0,h]
    e1 = nil
    e2 = nil
    base_edges.each do |edge|
        e2 = container.add_line edge.start.position, apex
        e2.soft = true
        e2.smooth = true
        if( e1 )
            container.add_face edge, e2, e1
        end
        e1 = e2
    end
    
    # Create the last side face
    edge = base_edges[0]
    container.add_face edge.start.position, edge.end.position, apex
    
end
def default_parameters
    defaults = {"r", 2.feet, "h", 4.feet}
    defaults
end
def translate_key(key)
    prompt = key
    case( key )
        when "r"
            prompt = "Radius"
        when "h"
            prompt = "Height"
    end
    prompt
end
end
#=============================================================================
class Torus < Parametric
def create_entities(data, container)
    r1 = data["r1"].to_l # small radius
    r2 = data["r2"].to_l # big radius
    n1 = 24
    n2 = 24
    # Compute the cross-section circle points
    pts = points_on_circle([r2, 0, 0], [0, -1, 0], r1, n1)
    
    # Now create a polygon mesh and revolve these points
    numpts = n1*n2
    numpoly = numpts
    mesh = Geom::PolygonMesh.new(numpts, numpoly)
    mesh.add_revolved_points(pts, [ORIGIN, Z_AXIS], n2)
    # create faces from the mesh
    container.add_faces_from_mesh( mesh, 12 )
    
end
def validate_parameters(data)
    ok = true
    # make sure that the small radius is less than the big radius
    if( data["r1"] >= data["r2"] )
        UI.messagebox "Inner radius must be smaller than outer radius"
        ok = false
    end
    ok
end
def default_parameters
    defaults = {"r1", 1.feet, "r2", 4.feet}
    defaults
end
def translate_key(key)
    prompt = key
    case( key )
        when "r1"
            prompt = "Inner Radius"
        when "r2"
            prompt = "Outer Radius"
    end
    prompt
end
end
#=============================================================================
class Tube < Parametric
def create_entities(data, container)
    ro = data["r"].to_l
    ri = ro - data["t"].to_l
    h = data["h"].to_l
    n = 24
    outer = container.add_circle ORIGIN, Z_AXIS, ro, n
    face = container.add_face outer
    inner = container.add_circle ORIGIN, Z_AXIS, ri, n
    inner[0].faces.each {|f| f.erase! if( f != face)}
    h = -h if face.normal.dot(Z_AXIS) < 0.0
    face.pushpull h
    
end
def default_parameters
    defaults = {"r", 2.feet, "t", 3.inch, "h", 4.feet}
    defaults
end
def validate_parameters(data)
    ok = true
    # make sure that the thickness is less than the radius
    if( data["t"] >= data["r"] )
        UI.messagebox "Thickness must be smaller than radius"
        ok = false
    end
    ok
end
def translate_key(key)
    prompt = key
    case( key )
        when "r"
            prompt = "Radius"
        when "t"
            prompt = "Wall Thickness"
        when "h"
            prompt = "Height"
    end
    prompt
end
end
#=============================================================================
class Pyramid < Parametric
def create_entities(data, container)
    b = data["b"].to_l
    h = data["h"].to_l
    # Create the base
    b2 = b / 2.0
    pts = []
    pts[0] = [-b2, -b2, 0]
    pts[1] = [b2, -b2, 0]
    pts[2] = [b2, b2, 0]
    pts[3] = [-b2, b2, 0]
    container.add_face pts
    
    # Create the sides
    apex = [0,0,h]
    container.add_face pts[0], apex, pts[1]
    container.add_face pts[1], apex, pts[2]
    container.add_face pts[2], apex, pts[3]
    container.add_face pts[3], apex, pts[0]
    
end
def default_parameters
    defaults = {"b", 4.feet, "h", 4.feet}
    defaults
end
def translate_key(key)
    prompt = key
    case( key )
        when "b"
            prompt = "Base"
        when "h"
            prompt = "Height"
    end
    prompt
end
end
#=============================================================================
class Dome < Parametric
def create_entities(data, container)
    r = data["r"].to_l
    n90 = data["n"].to_i
    smooth = 12  # smooth
    # compute a quater circle
    arcpts = []
    delta = Math::PI/(2*n90)
    for i in 0..n90 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcpts.push(Geom::Point3d.new(r*cosa, 0, r*sina))
    end
    
    # create a mesh and revolve the quater circle
    numpoly = n90*n90*4
    numpts = numpoly + 1
    mesh = Geom::PolygonMesh.new(numpts, numpoly)
    mesh.add_revolved_points(arcpts, [ORIGIN, Z_AXIS], n90*4)
    # create faces from the mesh
    container.add_faces_from_mesh( mesh, smooth )
    
end
def default_parameters
    defaults = {"r", 2.feet, "n", 5}
    defaults
end
def translate_key(key)
    prompt = key
    case( key )
        when "r"
            prompt = "Radius"
        when "n"
            prompt = "Segments(90)"
    end
    prompt
end
def validate_parameters(data)
    ok = true
        if( data["n"] < 1 )
        UI.messagebox "At least 1 segment required"
        ok = false
    end
    ok
end
end
#=============================================================================
###################################################################################################
####   
####        3D SHAPES TOOL
####
###################################################################################################
########################################################
#
#    3d Shapes Tool - Designed  May 2008 by Macs 
#
########################################################
# Copyright 2004-2005, @Last Software, Inc.
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name        :  3d shapes tool 1.0
# Description :   A tool to create varius 3d shapes with the mouse.
# Menu Item   :   Draw->3D Shapes->...
# Context Menu:   NONE
# Date        :   8/05/2008
# Type        :   Tool
#Based on  : Rotated Rectangle Tool 1.0
#-----------------------------------------------------------------------------

#=============================================================================
class PyramidTool
@@ltcursor = 0
@@toggleDrawingMode = 0
BYRADIUS = 0
BYSIDE = 1
def initialize
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
	@planex = [Geom::Point3d.new(0,0,0), Geom::Vector3d.new(1,0,0)]
	@planey	= [Geom::Point3d.new(0,0,0), Geom::Vector3d.new(0,1,0)]
	@planez = [Geom::Point3d.new(0,0,0), Geom::Vector3d.new(0,0,1)]
    if @@ltcursor == 0
		plugins = Sketchup.find_support_file("Plugins")	
		imgdir = File.join(plugins, "3DShapesToolbar")
		c = File.join(imgdir, "3dshapes_cursor.png")
		@@ltcursor = UI::create_cursor(c, 1, 30)
    end
    reset
end
def reset
    @moved = false
    @pts = []
	@pir_pts 
    @state = 0
	@vertex = 0
    @ip1.clear
    @drawn = false
    Sketchup::set_status_text "", SB_VCB_LABEL
    Sketchup::set_status_text "", SB_VCB_VALUE
    Sketchup::set_status_text "Click for start point"
    @shift_down_time = Time.now
end
def activate
    self.reset
end
def deactivate(view)
    view.invalidate if @drawn
end
#def resume(view)
#	view.invalidate
#end
def onSetCursor
	UI::set_cursor(@@ltcursor)
end
def set_current_point(x, y, view)
    if( !@ip.pick(view, x, y, @ip1) )
        return false
    end
    need_draw = true
    
    # Set the tooltip that will be displayed
    view.tooltip = @ip.tooltip
        
    # Compute points
    case @state
    when 0
        @pts[0] = @ip.position
        @pts[4] = @pts[0]
        need_draw = @ip.display? || @drawn
    when 1
        @pts[1] = @ip.position
        @width = @pts[0].distance @pts[1]
        Sketchup::set_status_text @width.to_s, SB_VCB_VALUE
    when 2	    
        pt1 = @ip.position
        pt2 = pt1.project_to_line @pts
        vec = pt1 - pt2
        @deep = vec.length
        if( @deep > 0 )
            # test for a square
            square_point = pt2.offset(vec, @width)
            if( view.pick_helper.test_point(square_point, x, y) )
                @deep = @width
                @pts[2] = @pts[1].offset(vec, @deep)
                @pts[3] = @pts[0].offset(vec, @deep)
                view.tooltip = "Square"
            else
                @pts[2] = @pts[1].offset(vec)
                @pts[3] = @pts[0].offset(vec)
            end
			self.temp_apex(@pts, view)
        else
            @pts[2] = @pts[1]
            @pts[3] = @pts[0]
        end
        Sketchup::set_status_text @deep.to_s, SB_VCB_VALUE
	when 3
		@pir_pts = self.get_apex(view,@ip,@vect_perp,@center)
		@height = @pir_pts.distance @center
		Sketchup::set_status_text @height.to_s, SB_VCB_VALUE
    end
    view.invalidate if need_draw
end
def onMouseMove(flags, x, y, view)
    @moved = true
    self.set_current_point(x, y, view)
end
def temp_apex( pts, view )
	@center = calculate_center(@pts)
	@height = view.pixels_to_model 50.cm, @ip.position
	vec1 = @pts[0] - @center
	vec2 = @center- @pts[1]
	@vect_perp = vec1.cross vec2		
	@pir_pts = @center.offset(@vect_perp, @height)
end
def calculate_center( pts )
	middlepoint = Geom::Point3d.linear_combination 0.5, pts[0], 0.5, pts[1]
	vec = pts[2] - pts[1]
	vec.length = vec.length / 2
	return middlepoint.offset(vec)
end
#get apex point
def get_apex(view, ip, vec, origin)
	if (ip.degrees_of_freedom != 3 )
		return ip.position.project_to_line([origin, vec])
	else
		@c_orig = view.screen_coords origin
		@c_ip = view.screen_coords ip.position
		@c_on_normal = view.screen_coords origin.offset( vec )
		@c_apex = @c_ip.project_to_line [@c_orig, @c_on_normal]
		a = Geom.closest_points [origin, vec], view.pickray(@c_apex.x, @c_apex.y)
		return a[0]
	end	
end
def create_geometry
    # check for zero deep
    if( @pts[0] != @pts[3] )
		model = Sketchup.active_model
		model.start_operation $exStrings.GetString("Pyramid")
        model.active_entities.add_face @pts
		for i in 0..3
			Sketchup.active_model.active_entities.add_face @pir_pts,@pts[i], @pts[i+1]
		end
		model.commit_operation
    end
    self.reset
end
def increment_state
	@moved = false
	@state += 1				
    case @state
    when 1
		@ip1.copy! @ip
		Sketchup::set_status_text "Click for second point"
		Sketchup::set_status_text "Length", SB_VCB_LABEL
		Sketchup::set_status_text "Move mouse", SB_VCB_VALUE
		@pts[1] = @pts[0]
    when 2
        @ip1.clear
        Sketchup::set_status_text "Click for third point"
        Sketchup::set_status_text "Width", SB_VCB_LABEL
        Sketchup::set_status_text "Move mouse", SB_VCB_VALUE
		@pts[2] = @pts[0]
		@pts[3] = @pts[0]
	when 3
        Sketchup::set_status_text "Click for fourth point"
        Sketchup::set_status_text "Height", SB_VCB_LABEL
        Sketchup::set_status_text "Move mouse", SB_VCB_VALUE	
    when 4
        self.create_geometry
    end
end
def onLButtonDown(flags, x, y, view)
    if @moved 
	    self.set_current_point(x, y, view)
	    self.increment_state
	    view.lock_inference
	end
end
def onCancel(flag, view)
    view.invalidate if @drawn
    self.reset
end
# This is called when the user types a value into the VCB
def onUserText(text, view)
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l # convert the value to a length
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    
    case @state
    when 1
        # update the width
        vec = @pts[1] - @pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            @pts[1] = @pts[0].offset(vec)
            view.invalidate
            self.increment_state
        end
    when 2
        # update the deep
        vec = @pts[3] - @pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            @pts[2] = @pts[1].offset(vec)
            @pts[3] = @pts[0].offset(vec)
			temp_apex(@pts, view)
            self.increment_state
        end
	when 3
		@pir_pts = @center.offset(@vect_perp, value)
		self.increment_state
    end
end
def getExtents
    bb = Sketchup.active_model.bounds
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? )
            bb.add @ip.position
        end
    when 1
        bb.add @pts[0]
        bb.add @pts[1]
    when 2
        bb.add @pts
		bb.add @pir_pts
	when 3
		bb.add @pir_pts	
    end
    bb
end
def draw(view)
    @drawn = false
    
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end
    # show the rectangle
    if( @state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line(@ip1, @ip)
        inference_locked = view.inference_locked?
        view.line_width = 3 if inference_locked
        view.draw(GL_LINE_STRIP, @pts[0], @pts[1])
        view.line_width = 1 if inference_locked
        @drawn = true
    elsif( @state > 1 )
        # draw the curve
        view.drawing_color = "black"
        view.draw(GL_LINE_STRIP, @pts)
		view.draw_lines @pir_pts, @pts[0], @pir_pts, @pts[1], @pir_pts, @pts[2], @pir_pts, @pts[3]
		view.drawing_color = "gray"		
		view.line_stipple="_"
		view.draw_lines(@pir_pts, @ip.position)		
        @drawn = true
    end
end
def onKeyDown(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && rpt == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            view.lock_inference
        elsif( @state == 0 )
            view.lock_inference @ip
        elsif( @state == 1 )
            view.lock_inference @ip, @ip1
        end
		
    end
	if( key == COPY_MODIFIER_KEY && rpt == 1 )
	   @@toggleDrawingMode +=1
	   @@toggleDrawingMode %=2
	   if ( @state == 1 )
			case @@toggleDrawingMode 
			when BYSIDE	     
				Sketchup::set_status_text "Click to set side length"
				Sketchup::set_status_text "Side", SB_VCB_LABEL
			when BYRADIUS
				Sketchup::set_status_text "Click to set radius length"
				Sketchup::set_status_text "Radius", SB_VCB_LABEL
			end
	   end					
	   view.invalidate
	end
	#hits F2, swicth vertex
	if ( @@toggleDrawingMode == BYSIDE && key == 113 )
		@vertex += 1
		@vertex %= @sides
		view.invalidate
	end
end
def onKeyUp(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end
end # of class PyramidTool
class ConeTool < PyramidTool
def reset
	if(@centerLine!=nil)
        Sketchup.active_model.abort_operation #get rid of cline
        @centerLine=nil
    end
    @moved = false
    @pts = []
	@pir_pts 
    @state = 0
	@vertex = 0	
    @ip1.clear
    @drawn = false
	@sides = 24
    Sketchup::set_status_text "Sides > 2", SB_VCB_LABEL
    Sketchup::set_status_text @sides, SB_VCB_VALUE
    Sketchup::set_status_text "Click for center of cone"
    @shift_down_time = Time.now	
end
def set_current_point(x, y, view)
    if( !@ip.pick(view, x, y, @ip1) )
        return false
    end
    need_draw = true
    
    # Set the tooltip that will be displayed
    view.tooltip = @ip.tooltip
        
    # Compute points
    case @state
    when 0
        @pts[0] = @ip.position
        need_draw = @ip.valid? || @drawn
    when 1
        @pts[1] = @ip.position
        @radius = @pts[0].distance @pts[1]
        Sketchup::set_status_text @radius.to_s, SB_VCB_VALUE
    when 2
		@pts[2] = self.get_apex(view, @ip, @normal, @pts[0])
		@height = @pts[0].distance @pts[2]
        Sketchup::set_status_text @height.to_s, SB_VCB_VALUE
    end
    view.invalidate if need_draw
end
def draw(view)
    @drawn = false   
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end
	return  if (!@ip.valid?) #this will eliminate the first draw on selecting the tool.
	
    # show the circle
	if( @state == 0 )
	    point = @ip.position
		@radius = view.pixels_to_model 50.cm, @ip.position
		@height = @radius*2 	
		if point.on_plane? @planey
			@normal = Geom::Vector3d.new(0,1,0)
			view.drawing_color = "green"
		elsif point.on_plane? @planex
			@normal = Geom::Vector3d.new(1,0,0)
			view.drawing_color = "red"			
		else 
			@normal = Geom::Vector3d.new(0,0,1)
			view.drawing_color = "blue"
		end
		face = @ip.face		
		if ( face )
		    @normal = face.normal
		end	
		pts_circle = self.points_on_circle( @ip.position,@normal, @radius, @sides, nil)
		apex = point.offset(@normal, @height)
		view.draw_lines(point, apex)
		for i in 0..@sides
			view.draw_lines(apex, pts_circle[i])
		end
		view.draw(GL_LINE_STRIP, pts_circle)
		@drawn = true
    elsif( @state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line(@ip1, @ip)
        #inference_locked = view.inference_locked?
        #view.line_width = 3 if inference_locked		
		#view.lock_inference @ip1
		vector = nil
		ptr = @ip.position		
		v1 = ptr - @pts[0]
		if (@normal == nil)
			ptr2 = ptr.clone
			ptr2.x = ptr.y
			ptr2.y = ptr.x			
			v2 = ptr2 - @pts[0]
			@normal = v1.cross v2
		else
			ptr = ptr.project_to_plane [@pts[0], @normal]
			v1 = ptr - @pts[0]
		end		
		pts_circle = self.points_on_circle( @pts[0], @normal, v1.length, @sides, v1)	
		view.draw(GL_LINE_STRIP, @pts[0], ptr)
		apex = @pts[0].offset(@normal, @height)
		view.drawing_color = "blue"
        view.draw(GL_LINE_STRIP, pts_circle)
		for i in 0..@sides
			view.draw_lines(apex, pts_circle[i])
		end
        @drawn = true
    elsif( @state > 1 )
        # draw the curve
		view.drawing_color = "blue"		
        pts_circle = self.points_on_circle( @pts[0], @normal, @radius, @sides, @pts[1]-@pts[0])		
		for i in 0..@sides
			view.draw_lines(@pts[2], pts_circle[i])
		end		
		view.draw(GL_LINE_STRIP, pts_circle)
		view.drawing_color = "gray"		
		view.line_stipple="_"
		view.draw_lines(@pts[2], @ip.position)
        @drawn = true
		#view.line_stipple=""
		#view.drawing_color = "red"		
		#view.draw2d GL_LINES, @c_orig, @c_ip
		#view.drawing_color = "green"
		#view.draw2d GL_LINES, @c_ip, @c_on_normal
		#view.drawing_color = "blue"
		#view.draw2d GL_LINES, @c_on_normal, @c_apex
		#view.drawing_color = "yellow"
		#view.draw2d GL_LINES, @c_apex, @c_orig
		
    end
end
def points_on_circle(center, normal, radius, numseg, v1)
	if (v1 == nil)
		axes = normal.axes
	    xaxis = axes[0]
	    yaxis = axes[1].reverse
	else
		axes = v1.axes
		if ( normal.parallel? v1 )
			xaxis = axes[1]
			yaxis = axes[2]
		else
			xaxis = normal.cross v1
			yaxis = axes[2]
		end
    end
	
    xaxis.length = radius
    yaxis.length = radius
    # compute the points
    da = (Math::PI * 2) / numseg
    pts = []
    for i in 0..numseg do
        angle = i * da 
        cosa = Math.cos(angle) 
        sina = Math.sin(angle) 
		vec= Geom::Vector3d.linear_combination(sina, xaxis, cosa, yaxis)
        pts.push(center + vec)
    end
    pts
end
def increment_state
	@moved = false
	@state += 1				
    case @state
    when 1
		@ip1.copy! @ip
		Sketchup::set_status_text "Click to set radius length"
		Sketchup::set_status_text "Radius", SB_VCB_LABEL
		Sketchup::set_status_text "Move mouse", SB_VCB_VALUE
    when 2
        @ip1.clear
		Sketchup.active_model.start_operation "Cone"		
		@centerLine = Sketchup.active_model.entities.add_cline(@pts[0], @normal)		
        Sketchup::set_status_text "Click to set the height"
        Sketchup::set_status_text "Height", SB_VCB_LABEL
        Sketchup::set_status_text "Move mouse", SB_VCB_VALUE
    when 3
        Sketchup.active_model.abort_operation #get rid of cline
        @centerLine=nil	
        self.create_geometry
    end
end
def getExtents
    bb = Sketchup.active_model.bounds
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? )
            bb.add @ip.position
        end
    when 1
        bb.add @pts[0]
		if (@pts[1] != nil)
			bb.add @pts[1]
		end
    when 2
        bb.add @pts
    end
    bb
end
# This is called when the user types a value into the VCB
def onUserText(text, view)
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
	
    
    case @state
    when 0
		return if ( text.to_i < 3 )
		@sides = text.to_i
		view.invalidate
    when 1
	    # update the radius
		@radius = value
		view.invalidate		
        self.increment_state
    when 2
        # update the height
		@height = value
		@pts[2]= @pts[0].offset(@normal, @height)
		self.increment_state
    end
end
def create_geometry
	model = Sketchup.active_model
	if @sides > 8
		model.start_operation $exStrings.GetString("Cone")
	else 
		model.start_operation $exStrings.GetString("Pyramid")
	end
	
    # Create the base
	ent = model.active_entities 
    circle = ent.add_circle @pts[0], @normal, @radius, @sides
	numf = circle[0].find_faces #this actually creates a face when needed
	if ( numf == 1 && @normal != [0,0,1] )
		circle[0].faces[0].reverse!
	end
    e2 = []
	for i in 0...@sides
		e2[i] = ent.add_line(circle[i].start.position, @pts[2])
		if @sides > 8
			e2[i].soft = true
			e2[i].smooth = true
		end
	end
	for i in 0...@sides-1
        ent.add_face circle[i], e2[i], e2[i+1]
	end
	ent.add_face circle[@sides-1], e2[@sides-1], e2[0]
	model.commit_operation	
    self.reset
end
end # end ConeTool
class SphereTool < ConeTool
def reset
    @moved = false
    @pts = []
    @state = 0
    @ip1.clear
    @drawn = false
	@vertex = 0
	@sides = 24
    Sketchup::set_status_text "Sides > 3", SB_VCB_LABEL
    Sketchup::set_status_text @sides, SB_VCB_VALUE
    Sketchup::set_status_text "Hit CTRL to toggle between drawing modes"
    @shift_down_time = Time.now
end
def increment_state
	@moved = false
	@state += 1				
    case @state
    when 1
		@ip1.copy! @ip
		Sketchup::set_status_text "Click to set radius length"
		Sketchup::set_status_text "Radius", SB_VCB_LABEL
		Sketchup::set_status_text "Move mouse", SB_VCB_VALUE
    when 2
		@ip1.clear
        self.create_geometry
    end
end
def points_on_circle(center, normal, radius, numseg)
    # Get the x and y axes
    #axes = Geom::Vector3d.new(normal).axes
	axes = normal.axes
    #center = Geom::Point3d.new(center)
    xaxis = axes[0]
    yaxis = axes[1]
    
    xaxis.length = radius
    yaxis.length = radius
    # compute the points
    da = (Math::PI * 2) / numseg
    pts = []
    for i in 0..numseg do
        angle = i * da
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        #vec = Geom::Vector3d.linear_combination(cosa, xaxis, sina, yaxis)
		vec = Geom::Vector3d.linear_combination(sina, xaxis, cosa, yaxis)
        pts.push(center + vec)
    end
    
    # close the circle
    #pts.push(pts[0].clone)
    pts
end
def set_current_point(x, y, view)
    if( !@ip.pick(view, x, y, @ip1) )
        return false
    end
    need_draw = true
    
    # Set the tooltip that will be displayed
    view.tooltip = @ip.tooltip
        
    # Compute points
    case @state
    when 0
        @pts[0] = @ip.position
        need_draw = @ip.valid? || @drawn
    when 1
        @pts[1] = @ip.position
        need_draw = @ip.valid? || @drawn		
        @radius = @pts[0].distance @pts[1]
        Sketchup::set_status_text @radius.to_s, SB_VCB_VALUE		
    end	
    view.invalidate if need_draw
end
def draw(view)
    @drawn = false   
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end
	
	return if (!@ip.valid?)
		
    # show the circle
	if( @state == 0 )
	    point = @ip.position
		@radius = view.pixels_to_model 100.cm, @ip.position			
		if point.on_plane? @planey
			@normal = Geom::Vector3d.new(0,1,0)
			view.drawing_color = "green"
		elsif point.on_plane? @planex
			@normal = Geom::Vector3d.new(1,0,0)
			view.drawing_color = "red"			
		else 
			@normal = Geom::Vector3d.new(0,0,1)
			view.drawing_color = "blue"
		end
		face = @ip.face		
		if ( face )
		    @normal = face.normal
		end	
		pts_circle = self.points_on_circle( point,Geom::Vector3d.new(0,0,1), @radius, @sides)		
		
		@t = Geom::Transformation.new 
		if ( !(@normal.parallel? Geom::Vector3d.new(0,0,1)) ) 
			vector = Geom::Vector3d.new(0,0,1).cross @normal
			angle = @normal.angle_between Geom::Vector3d.new(0,0,1)
			@t = Geom::Transformation.rotation point,vector, angle
		end
		case @@toggleDrawingMode
		when BYSIDE 		    
			#first draw the old one
			for x in 0...pts_circle.size
				pts_circle[x].transform!(@t)
			end
			view.draw(GL_LINE_STRIP, pts_circle)			
			point.offset!( Geom::Vector3d.new(0,0,1), @radius )			
			pts_circle = self.points_on_circle( point,Geom::Vector3d.new(0,0,1), @radius, @sides)			
		end
		
		base = []
		top = []	
		for i in 0..@sides
			va = point - pts_circle[i]
			pts_circle[i].transform! @t
			#vdot = va.axes[0].reverse!
			pts_circle2 = self.points_on_semic( point,va, @radius, @sides)			
			for x in 0...pts_circle2.size
				pts_circle2[x].transform! @t
			end
			view.draw(GL_LINE_STRIP, pts_circle2)		
			base.push(pts_circle2[0])
			top.push(pts_circle2.last)
		end
		view.draw(GL_LINE_STRIP, top)
		view.draw(GL_LINE_STRIP, base)
		view.draw(GL_LINE_STRIP, pts_circle)							
		#pts_circle = self.points_on_circle( point,Geom::Vector3d.new(0,1,0), @radius, @sides)
		#view.draw(GL_LINE_STRIP, pts_circle)
		
		#pts_circle = self.points_on_circle( point,Geom::Vector3d.new(1,0,0), @radius, @sides)
		#view.draw(GL_LINE_STRIP, pts_circle)
		
		@drawn = true
    elsif( @state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line(@ip1, @ip)
		ptr = @ip.position		
		return if (ptr == @pts[0])
		
		point = @pts[0].clone		
		view.draw(GL_LINE_STRIP, point, ptr)
		view.drawing_color = "blue"		
		v1 = ptr - point
		@radius = v1.length
		face = @ip.face		
		if ( face )
		    @normal = face.normal
		end			
		
		#pts_circle = self.points_on_circle( @pts[0],@normal, @radius, @sides)
		pts_circle = self.points_on_circle( point ,Geom::Vector3d.new(0,0,1), @radius, @sides)	
				
		@t = Geom::Transformation.new 
		if ( !(@normal.parallel? Geom::Vector3d.new(0,0,1))  ) 
			vector = Geom::Vector3d.new(0,0,1).cross @normal
			angle = @normal.angle_between Geom::Vector3d.new(0,0,1)
			@t = Geom::Transformation.rotation point,vector, angle
		end
		case @@toggleDrawingMode
		when BYSIDE 		    
			for x in 0...pts_circle.size
				pts_circle[x].transform!(@t)
			end
			view.draw(GL_LINE_STRIP, pts_circle)			
			point.offset!( Geom::Vector3d.new(0,0,1), @radius )			
			pts_circle = self.points_on_circle( point,Geom::Vector3d.new(0,0,1), @radius, @sides)		
		end
		
		#view.drawing_color = "red"
		base = []
		top = []	
		for i in 0..@sides
			va = point - pts_circle[i] 
			pts_circle[i].transform! @t
			#vdot = va.axes[0].reverse!
			#pdot = @pts[0].offset(vdot, 200)
			#view.draw_lines(@pts[0], pdot )			
			pts_circle2 = self.points_on_semic( point,va, @radius, @sides)
			for x in 0...pts_circle2.size
				pts_circle2[x].transform! @t
			end			
			view.draw(GL_LINE_STRIP, pts_circle2)
			base.push(pts_circle2[0])
			top.push(pts_circle2.last)
			#view.drawing_color = "green"			
		end
		view.draw(GL_LINE_STRIP, pts_circle)		
		view.draw(GL_LINE_STRIP, top)
		view.draw(GL_LINE_STRIP, base)
		#view.drawing_color = "red"
		#pts_circle = self.points_on_circle( @pts[0],Geom::Vector3d.new(0,1,0), @radius, @sides)
		#view.draw(GL_LINE_STRIP, pts_circle)
        @drawn = true
    end
end
def points_on_semic(center, normal, radius, numseg)
    # Get the x and y axes
	zaxes = Geom::Vector3d.new(0,0,1)
	rad = normal.angle_between  zaxes
	
    axes = Geom::Vector3d.new(normal).axes
    #center = Geom::Point3d.new(center)
    xaxis = axes[0]
    yaxis = axes[1]
    
    xaxis.length = radius
    yaxis.length = radius
	#numseg = numseg*2 if (numseg % 2 == 1)	
	
    # compute the points
    da = (Math::PI * 2) / (numseg)
    pts = []
    for i in -(numseg/4)..numseg/4 do
	#for i in 0..numseg/2 do
        angle = i * da + rad
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        #vec = Geom::Vector3d.linear_combination(cosa, xaxis, sina, yaxis)
		vec = Geom::Vector3d.linear_combination(sina, xaxis, cosa, yaxis)
        pts.push(center + vec)
    end
   
    pts
end
# This is called when the user types a value into the VCB
def onUserText(text, view)
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
	
    
    case @state
    when 0
		return if ( text.to_i < 4 )
		@sides = text.to_i
		view.invalidate
    when 1
	    # update the radius
		@radius = value
		view.invalidate		
        self.increment_state
    end
end
def create_geometry
	model = Sketchup.active_model
	if @sides > 12
		model.start_operation $exStrings.GetString("Sphere")
	else 
		model.start_operation $exStrings.GetString("Polyhedron")
	end
	
    # Create the base
	ent = model.active_entities 
	case @@toggleDrawingMode
	when BYSIDE 		    
		@pts[0].offset!( Geom::Vector3d.new(0,0,1), @radius )			
	end	
    pts_circle = self.points_on_circle( @pts[0],Geom::Vector3d.new(0,0,1), @radius, @sides)
	#circle = ent.add_circle( @pts[0],Geom::Vector3d.new(0,0,1), @radius, @sides)
	base = []
	top = []
	pts_circle2 = []
	
	for i in 0..@sides
		va = @pts[0] - pts_circle[i]
		#vdot = va.axes[0].reverse!
		pts_circle2[i] = self.points_on_semic( @pts[0],va, @radius, @sides)
		base.push(pts_circle2[i][0].transform(@t))
		top.push(pts_circle2[i][pts_circle2[i].size-1].transform(@t))
	end
	
	
	s = pts_circle2[i].size
	draw_top = draw_base = true
	for x in 0...(s-1)
		for i in 0...@sides		
			a = pts_circle2[i][x].transform @t
			b = pts_circle2[(i+1)%@sides][x].transform @t
			c = pts_circle2[(i+1)%@sides][(x+1)%s].transform @t
			d = pts_circle2[i][(x+1)%s].transform @t
			if ( a == b )
				face = ent.add_face a, c, d
				draw_top = false
			elsif ( c == d )
				face = ent.add_face a, b, d
				draw_base = false
			else
				face = ent.add_face a, b, c, d
			end
			#if the first face draw is on a face we have to reverse it or all the polihedron will get reversed
			#if ( x == 0 )
			#	edges = face.edges
			#	reversed = edges[0].reversed_in? face
			#	face.reverse! if ( reversed )
			#end
			
			if (@sides >= 12)
				ed = face.edges 
				ed[0].soft=true
				ed[1].soft=true
				ed[2].soft=true
				ed[0].smooth=true
				ed[1].smooth=true
				ed[2].smooth=true
			end
		end
	end	
	ent.add_face(top)  if draw_top 
	ent.add_face(base) if draw_base
	model.commit_operation	
    self.reset
end
def getExtents
    bb = Sketchup.active_model.bounds
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? )
            bb.add @ip.position
        end
    when 1
        bb.add @pts
        if( @ip.valid? )
            bb.add @ip.position
        end
	end
    bb
end
end # SphereTool
class DodecahedronTool < ConeTool
def reset
    @moved = false
    @pts = []
	@pt = []
    @state = 0
    @ip1.clear
    @drawn = false
	@vertex = 0
	@sides = 5
    Sketchup::set_status_text "", SB_VCB_LABEL
    Sketchup::set_status_text @sides, SB_VCB_VALUE
    Sketchup::set_status_text "Hit CTRL to toggle between drawing modes"
    @shift_down_time = Time.now
end
def increment_state
	@moved = false
	@state += 1				
    case @state
    when 1
		@ip1.copy! @ip
		Sketchup::set_status_text "Click to set radius length"
		Sketchup::set_status_text "Radius", SB_VCB_LABEL
		Sketchup::set_status_text "Move mouse", SB_VCB_VALUE
    when 2
		@ip1.clear
        self.create_geometry
    end
end
def draw(view)
    @drawn = false   
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end
	
	return if (!@ip.valid?)
		
    # show the circle
	if( @state == 0 )
	    point = @ip.position
		@radius = view.pixels_to_model 100.cm, @ip.position
		if point.on_plane? @planey
			@normal = Geom::Vector3d.new(0,1,0)
			drawing_color = "green"
		elsif point.on_plane? @planex
			@normal = Geom::Vector3d.new(1,0,0)
			drawing_color = "red"			
		else 
			@normal = Geom::Vector3d.new(0,0,1)
			drawing_color = "blue"
		end
		face = @ip.face		
		if ( face )
		    @normal = face.normal
		end	
		#pts_circle = self.points_on_circle( point,@normal, @radius, @sides, nil)
		case @@toggleDrawingMode
		when BYSIDE 
			@radius = self.calc_radius_from_side( @radius ) 		
			pts_circle = self.points_on_circle( point,@normal, @radius, @sides, nil)	
			rad = 54.degrees
			move_vector = point - pts_circle[0]
			translation = Geom::Transformation.translation move_vector
			rotation = Geom::Transformation.rotation point, @normal, rad
			@t = rotation * translation #first rotate, then translate.. inverse different results
			for i in 0...pts_circle.size
			   pts_circle[i] = @t * pts_circle[i] 
			end
		when BYRADIUS
			pts_circle = self.points_on_circle( point,@normal, @radius, @sides, nil)	
		end
		view.drawing_color = "cyan"
		view.draw(GL_LINE_STRIP,pts_circle)
		view.drawing_color = drawing_color
		
=begin debug code		
		angle = 116.565
		rad = (angle * Math::PI/180.0)
		pts_p = []		
		pts_y = []
		@pt = []
		for i in 0...@sides		
			va = pts_circle[(i+1)%@sides] - pts_circle[i] 
			transform = Geom::Transformation.rotation pts_circle[i], va, rad			
			pts_p = []
			for x in 0..@sides 
				pts_p[(((x-i)+@sides)%@sides)] = pts_circle[x].transform transform 		
			end
			view.drawing_color = "red"
			view.draw(GL_LINE_LOOP, pts_p)
			vb = pts_p[3] - pts_p[4]
			#vbdot = pts_p[1].offset(vb, 200.0)
			view.drawing_color = "blue"
			view.draw_lines(pts_p[3],pts_p[4] )	
			t2 = Geom::Transformation.rotation pts_p[3], vb, rad		
			for y in 0...@sides
				pts_y[y] = pts_p[y].transform t2
			end
			view.drawing_color = "green"
			view.draw(GL_LINE_LOOP, pts_y)
			#@pt += pts_p
			#@pt += pts_y
		end
=end
		@pt = self.calc_points( pts_circle )
		for x in 0...(@sides*2)
		    #puts (@sides*x)
			#puts (@sides+(@sides*x-1))
			view.draw(GL_LINE_STRIP, @pt[(@sides*x)..(@sides+(@sides*x-1))])
		end
		@drawn = true
    elsif( @state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line(@ip1, @ip)
		point = @ip.position		
		return if (point == @pts[0])
		
		view.draw(GL_LINE_STRIP, @pts[0], point)		
		v1 = point - @pts[0]
		@radius = v1.length	
		face = @ip.face		
		if ( face )
		    @normal = face.normal
		end		
		
		#@normal = v1.axes[0]
		#view.drawing_color = "green"		
		#view.draw_lines( point, point.offset(v1.axes[1], 100))
		#view.drawing_color = "red"		
		#view.draw_lines( point, point.offset(v1.axes[2], 100))	
		#pts_circle = self.points_on_circle( @pts[0],@normal, @radius, @sides, v1)
		
		case @@toggleDrawingMode
		when BYSIDE 
			@radius = self.calc_radius_from_side( @radius ) 		
			pts_circle = self.points_on_circle( point,@normal, @radius, @sides, v1)	
			rad = -54.degrees
			move_vector = point - pts_circle[0]
			translation = Geom::Transformation.translation move_vector
			rotation = Geom::Transformation.rotation point, @normal, rad
			@t = rotation * translation #first rotate, then translate.. inverse different results
			for i in 0...pts_circle.size
			   pts_circle[i] = @t * pts_circle[i] 
			end
		when BYRADIUS
			@radius = v1.length		
			pts_circle = self.points_on_circle( @pts[0],@normal, @radius, @sides, v1)	
		end
		view.drawing_color = "cyan"
		view.draw(GL_LINE_STRIP,pts_circle)
		view.drawing_color = "blue"		
		
		
		@pt = self.calc_points( pts_circle )
		for x in 0...@sides*2
			view.draw(GL_LINE_STRIP, @pt[(@sides*x)..(@sides+(@sides*x-1))])
		end
		#view.draw_points(@pt[6],15,2,"pink")
		#view.draw_points(@pt[16],15,2,"white")
		#view.draw_points(@pt[26],15,2,"red")  
		#view.draw_points(@pt[36],15,2,"green")
		#view.draw_points(@pt[46],15,2,"yellow") 	
        @drawn = true
    end
end
def calc_points ( pts_circle )
	pts_p = []		
	pts_y = []
	angle = 116.56505
	#rad = (angle * Math::PI/180.0)
	rad = angle.degrees
	pt = []
	for i in 0...@sides		
		va = pts_circle[(i+1)%@sides] - pts_circle[i] 
		#mirroring the shape on one of his sides at a specific angle
		transform = Geom::Transformation.rotation pts_circle[i], va, rad			
		for x in 0..@sides
		    # rotare index to mantain points at the same places
			pts_p[(((x-i)+@sides)%@sides)] = pts_circle[x].transform transform 		
		end
		
		vb = pts_p[3] - pts_p[4]
		t2 = Geom::Transformation.rotation pts_p[3], vb, rad		
		for y in 0...@sides
			pts_y[y] = pts_p[y].transform t2
		end
		pt += pts_p
		pt += pts_y			
	end
	pt
end
# This is called when the user types a value into the VCB
def onUserText(text, view)
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
	
    
    case @state
    when 1
	    # update the radius
		case @@toggleDrawingMode 
		when BYSIDE		
			@radius = self.calc_radius_from_side( value )
		when BYRADIUS 
			@radius = value
		end
		view.invalidate		
        self.increment_state
    end
end
def getExtents
    bb = Sketchup.active_model.bounds
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? )
            bb.add @ip.position
		end
		
    when 1
        bb.add @pts
        if( @ip.valid? )
            bb.add @ip.position
			bb.add @pt
        end
	end
    bb
end
def create_geometry
	model = Sketchup.active_model
	model.start_operation $exStrings.GetString("Dodecahedron")
    # Create the base
	ent = model.active_entities 
	case @@toggleDrawingMode 
	when BYSIDE
		pts_circle = self.points_on_circle( @pts[1],@normal, @radius, @sides, @pts[1]-@pts[0])	
		for i in 0...pts_circle.size
			pts_circle[i] = @t * pts_circle[i] 
		end	
	when BYRADIUS
		pts_circle = self.points_on_circle( @pts[0],@normal, @radius, @sides, @pts[1]-@pts[0])	
	end	
	
	
	@pt = self.calc_points( pts_circle )
	top = []
	for x in 0...@sides
	   #puts (@sides+1+(@sides*x*2))
		top[x] = @pt[@sides+1+(@sides*x*2)]
	end	
	face = ent.add_face top #top
	#if the first face draw is on a face we have to reverse it or all the polihedron will get reversed
	edges = face.edges
	reversed = edges[0].reversed_in? face
	face.reverse! if ( reversed )
	
	for x in 0...(@sides*2)
		ent.add_face @pt[(@sides*x)..(@sides+(@sides*x-1))]
	end
	ent.add_face pts_circle #bottom
	
	model.commit_operation	
    self.reset
end
def sphere_radius( side )
	r = side * 1.401258539
	#r = Math.sqrt(3)*phi/2 * side
end
def side_from_sp_radius ( radius )
	#r = side * 
	#r = 2/(Math.sqrt(3)*phi) *radius
	r = 0.713644180 * radius
end
#radius of the circle enclosing one of the pentagon
def calc_radius_from_side ( s )
    #radius = side * Math.sqrt(2+2/Math.sqrt(5))/2
	#0,85065080835203993218154049706301
	side = s.to_f
	radius = side * 0.850650808
	radius.to_l
end
end #DodecahedronTool
class IcosahedronTool < DodecahedronTool
def reset
    @moved = false
    @pts = []
	@pt = []
    @state = 0
    @ip1.clear
    @drawn = false
	@vertex = 0
	@sides = 3
    Sketchup::set_status_text "", SB_VCB_LABEL
    Sketchup::set_status_text @sides, SB_VCB_VALUE
    Sketchup::set_status_text "Hit CTRL to toggle between drawing modes"
    @shift_down_time = Time.now
end
def draw(view)
    @drawn = false   
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end
	
	return if (!@ip.valid?)
		
    # show the circle
	if( @state == 0 )
	    point = @ip.position
		@radius = view.pixels_to_model 100.cm, @ip.position
		if point.on_plane? @planey
			@normal = Geom::Vector3d.new(0,1,0)
			view.drawing_color = "green"
		elsif point.on_plane? @planex
			@normal = Geom::Vector3d.new(1,0,0)
			view.drawing_color = "red"			
		else 
			@normal = Geom::Vector3d.new(0,0,1)
			view.drawing_color = "blue"
		end
		face = @ip.face		
		if ( face )
		    @normal = face.normal
		end	
		#pts_circle = self.points_on_circle( point,@normal, @radius, @sides, nil)
		case @@toggleDrawingMode
		when BYSIDE 
			pts_circle = self.points_on_circle( point,@normal, @radius, @sides, nil)	
			rad = 30.degrees
			move_vector = point - pts_circle[@vertex]
			translation = Geom::Transformation.translation move_vector
			rotation = Geom::Transformation.rotation point, @normal, rad
			@t = rotation * translation #first rotate, then translate.. inverse different results
			for i in 0...pts_circle.size
			   pts_circle[i] = @t * pts_circle[i] 
			end
		when BYRADIUS
			pts_circle = self.points_on_circle( point,@normal, @radius, @sides, nil)	
		end
		
		view.drawing_color = "cyan"
		view.draw(GL_LINE_STRIP, pts_circle)
		view.drawing_color = "blue"				
=begin		
		angle = 138.189685
		rad = (angle * Math::PI/180.0)
		pts_p = []
		pts_y = []
		pts_x = []
		for i in 0...@sides		
			pts_p = calc_sides(pts_circle, i-1, rad,1)
			view.drawing_color = "blue"				
			view.draw(GL_LINE_STRIP, pts_p)			
			pts_y = calc_sides(pts_p, i, rad,0)
			view.drawing_color = "red"				
			view.draw(GL_LINE_STRIP, pts_y)			
			
			pts_x = calc_sides(pts_y, i+1, rad,1)
			view.drawing_color = "white"				
			view.draw(GL_LINE_STRIP, pts_x)			
			pts_y = calc_sides(pts_p, i+1, rad,0)
			view.drawing_color = "green"				
			view.draw(GL_LINE_STRIP, pts_y)			
			pts_x = calc_sides(pts_y, i, rad,1)
			view.drawing_color = "pink"				
			view.draw(GL_LINE_STRIP, pts_x)			
			
			pts_y = calc_sides(pts_x, i-1, rad,0)
			view.drawing_color = "yellow"				
			view.draw(GL_LINE_STRIP, pts_y)			
		end
		#puts "----"
=end
		
		@pt = self.calc_points( pts_circle )
		for x in 0...18
			view.draw(GL_LINE_STRIP, @pt[(@sides*x+x)..(@sides+(@sides*x)+x)])
		end
		
		@drawn = true
    elsif( @state == 1 )
        # just draw a line from the start to the end point		
		point = @ip.position				
	
		return if (point == @pts[0])
		v1 = point - @pts[0]
		
		face = @ip.face		
		if ( face )
		    @normal = face.normal
		end			
		
		case @@toggleDrawingMode
		when BYSIDE 
			@radius = self.calc_radius_from_side( @radius ) 		
			pts_circle = self.points_on_circle( point,@normal, @radius, @sides, v1)	
			rad = -30.degrees + (-60.degrees * @vertex * @vertex)
			move_vector = point - pts_circle[@vertex]
			translation = Geom::Transformation.translation move_vector
			rotation = Geom::Transformation.rotation point, @normal, rad
			@t = rotation * translation #first rotate, then translate.. inverse different results
			for i in 0...pts_circle.size
			   pts_circle[i] = @t * pts_circle[i] 
			end
		when BYRADIUS
			@radius = v1.length		
			pts_circle = self.points_on_circle( @pts[0],@normal, @radius, @sides, v1)	
		end
		
		view.drawing_color = "cyan"
		view.draw(GL_LINE_STRIP, pts_circle)
		
		#sr = self.sphere_radius( v1.length )
		#pts_sphere = self.points_on_circle( point,@normal, sr, 24, nil)				
		#view.draw(GL_LINE_STRIP, pts_sphere)	
		
		view.drawing_color = "blue"				
		@pt = self.calc_points( pts_circle )
		for x in 0...18
			view.draw(GL_LINE_STRIP, @pt[(@sides*x+x)..(@sides+(@sides*x)+x)])
		end
        view.set_color_from_line(@ip1, @ip)		
		view.draw(GL_LINE_STRIP, @pts[0], point)	
		
        @drawn = true
    end
end
def calc_radius_from_side( side_length )
	s = side_length
	h = Math.sqrt(3) * s.to_f / 2.0
	radius = h * 2 / 3
	radius.to_l
end
def calc_points ( pts_circle )
	angle = 138.189685
	#rad = (angle * Math::PI/180.0)
	rad = angle.degrees
	pts_p = []
	pts_y = []
	pts_x = []
	pt = []
	for i in 0...@sides		
		pt += pts_p = calc_sides(pts_circle, i-1, rad,1)
		pt += pts_y = calc_sides(pts_p, i, rad,0)
		pt += pts_x = calc_sides(pts_y, i+1, rad,1)
		pt += pts_y = calc_sides(pts_p, i+1, rad,0)
		pt += pts_x = calc_sides(pts_y, i, rad,1)
		pt += pts_y = calc_sides(pts_x, i-1, rad,0)
	end
	pt
end
def calc_sides( pts, index, rad, orient )
	newpts = []
	
	vb =  pts[((index+orient)+@sides)%@sides] - pts[((index+1-orient)+@sides)%@sides]
	t2 = Geom::Transformation.rotation pts[index], vb, rad		
	for y in 0..@sides
		newpts[y] = pts[y].transform t2
	end
	newpts
end
# This is called when the user types a value into the VCB
def onUserText(text, view)
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
	
    
    case @state
    when 1
	    # update the radius
		
		case @@toggleDrawingMode 
		when BYSIDE		
			@radius = self.calc_radius_from_side( @radius )
		when BYRADIUS 
			@radius = value
		end		
		view.invalidate		
        self.increment_state
    end
end
def getExtents
    bb = Sketchup.active_model.bounds
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? )
            bb.add @ip.position
		end
		
    when 1
        bb.add @pts
        if( @ip.valid? )
            bb.add @ip.position
			bb.add @pt
        end
	end
    bb
end
def create_geometry
	model = Sketchup.active_model
	model.start_operation $exStrings.GetString("Icosahedron")
	
    # Create the base
	ent = model.active_entities 
	
	case @@toggleDrawingMode 
	when BYSIDE
		pts_circle = self.points_on_circle( @pts[1],@normal, @radius, @sides, @pts[1]-@pts[0])	
		for i in 0...pts_circle.size
			pts_circle[i] = @t * pts_circle[i] 
		end	
	when BYRADIUS
		pts_circle = self.points_on_circle( @pts[0],@normal, @radius, @sides, @pts[1]-@pts[0])	
	end
	
	@pt = self.calc_points( pts_circle )
	
	top=[]
    top.push @pt[21]	
	top.push @pt[46]
	top.push @pt[71]
	face = ent.add_face top #top
	
	#if the first face draw is on a face we have to reverse it or all the polihedron will get reversed
	edges = face.edges
	reversed = edges[0].reversed_in? face
	face.reverse! if ( reversed )
	#now draw the rest of the faces...
	for x in 0...18
		face = ent.add_face @pt[(@sides*x+x)..(@sides+(@sides*x)+x)]
		edges = face.edges
	end
	#...and the bottom one
	face = ent.add_face pts_circle
	model.commit_operation	
    self.reset
end
def sphere_radius( side )
	r = side * 0.9510565163
	#r = Math.sqrt(phi**2 + 1) / 2 * side
end
def side_from_sp_radius ( radius )
	s = 1.0514622242 * radius
	#phi = 1.6180339887
	#s = 2 / Math.sqrt(phi**2 + 1) * radius
end 
end #IcosahedronTool
#=============================================================================
###################################################################################################
####   
#### 
####
###################################################################################################
###################################################################################################
####   
####                             SELECTION MEMORY
####
###################################################################################################
require "sketchup.rb"
module JF
  def self.sel_add
    @sel ||= []
    sel  = Sketchup.active_model.selection.collect
    sel.each { |s|
      @sel << s if s.parent == Sketchup.active_model
    }
    @sel.flatten!
    @sel.uniq!
    Sketchup.set_status_text("Selection: #{@sel.size}") 
  end
  def self.sel_sub
    sel = Sketchup.active_model.selection.collect
    @sel = @sel - sel
    Sketchup.active_model.selection.remove(sel)
    Sketchup.set_status_text("Selection: #{@sel.size}") 
  end
  def self.sel_recall
    @sel.reject! { |e| ! e.valid? }
    unless @sel
      Sketchup.set_status_text("Selection empty.")
      return
    end
    return unless @sel.size > 0
    Sketchup.active_model.selection.clear
    Sketchup.active_model.selection.add(@sel)
    Sketchup.set_status_text("Selection: #{@sel.size}") 
  end
end #module JF
#fn = "sel_mem.rb"
#unless file_loaded?(fn)
#  t = UI::Toolbar.new("Selection Memory")
#  cmd = UI::Command.new("Selection Add") { JF::sel_add }
#  cmd.small_icon = "selection_memory/s+16.png"
#  cmd.large_icon = "selection_memory/s+24.png"
#  cmd.tooltip = "Add Selected"
#  t.add_item cmd
#  cmd = UI::Command.new("Selection Subtract") { JF::sel_sub }
#  cmd.small_icon = "selection_memory/s-16.png"
#  cmd.large_icon = "selection_memory/s-24.png"
#  cmd.tooltip = "Subtract Selected"
#  t.add_item cmd
#  cmd = UI::Command.new("Selection Recall") { JF::sel_recall }
#  cmd.small_icon = "selection_memory/sr16.png"
#  cmd.large_icon = "selection_memory/sr24.png"
#  cmd.tooltip = "Recall Selected"
#  t.add_item cmd
  #UI.menu("Plugins").add_item("Remember Selection") { JF::selmem }
  #UI.menu("Plugins").add_item("Restore Selection") { JF::selresel }
#  file_loaded(fn)
#end
###################################################################################################
####   
####     COMPONENT REPORTER
####
###################################################################################################
=begin
#-----------------------------------------------------------------------
Copyright 2007 TIG
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided something the 
above copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------
Name        : 	ComponentReporter+.rb#
Type        : 	Tool
Description : 	Makes a report all components placed in a model.
Menu Item   : 	Plugins >> Component+ Report
Usage       : 	Pick and a report is written into a file called 
				XXX-ComponentReport+.csv for the Component Definitions.
				another report called XXX-Instance+Report.csv list info
				on individual palced components is also made.
				The reports are found in the model's folder.  
				They are readable by Excel and similar programs.
Version     :	1.0 20071022	First Issue.
#-----------------------------------------------------------------------
=end

### do class
class Reporter
### do def
 def Reporter::components_plus
   model=Sketchup.active_model
   entities=model.entities
   mname=model.title
### start undo...
   model.start_operation("Component+ Report")
### show VCB and status info...
   Sketchup::set_status_text(("COMPONENT+ REPORTER..." ), SB_PROMPT)
   Sketchup::set_status_text(" ", SB_VCB_LABEL)
   Sketchup::set_status_text(" ", SB_VCB_VALUE)
### check model...
   mpath=Dir::pwd
   if mpath==""
      UI.messagebox("You must save the 'Untitled' new model \nbefore making a Component+ Report !\nExiting... ")
      return nil
   end
### setup cdefn list...
   clist=[]
   model.definitions.each{|c|
      if c.count_instances > 0 and not c.group?
        clist.push([c.name,c.count_instances.to_s,c.description,c.guid])
      end#if
   } 
   clist.sort!
### if no components exit...
   if not clist[0]
      UI.messagebox("No Components available to make a Component+ Report !\nExiting... ")
      return nil
   end#if
### write to csv file...
   ccsv=mpath + "/" + mname + "-Component+Report.csv"
   file=File.new(ccsv,"w")
   file.puts("DEF-NAME,NUMBER,DESCRIPTION,GUID\n\n") 
   ### title ### add what you want to list here
   clist.each{|c|
     file.puts(c[0]+","+c[1]+","+c[2]+","+c[3])
   }
   file.close
### NOW make a list of all "instances" with details
### setup instances list...
   inlist=[]
   entities.each{|c|
      inlist.push(c) if c.typename=="ComponentInstance" and not c.definition.group?
   } 
   ilist=[]
   inlist.each{|c|
     dname="'"+c.definition.name+"'"###avoids loss of front zeros
	 iname="'"+c.name+"'"###avoids loss of front zeros
	 lname="'"+c.layer.name+"'"###avoids loss of front zeros
	 mat=c.material
	 if mat
	   cname="'"+mat.name+"'"###avoids loss of front zeros
	 else
	   cname="'<Default>'"###avoids loss of front zeros
	 end#if
	 area=0.0
	 c.definition.entities.each{|e|area=area+e.area if e.typename=="Face"}
	 area=area.to_s
	 ### this is in sq inches ### make a tweak here to suit your units
	 ### e.g. area=(area/144).to_s for sq ft
	 id=c.entityID.to_s
	 x=(c.bounds.max.x-c.bounds.min.x).to_l.to_s
	 y=(c.bounds.max.y-c.bounds.min.y).to_l.to_s
	 z=(c.bounds.max.z-c.bounds.min.z).to_l.to_s
     ilist.push([dname,iname,lname,cname,area,id,x,y,z])
   }
   ilist.sort!
### write to csv file...
   icsv=mpath + "/" + mname + "-Instance+Report.csv"
   file=File.new(icsv,"w")
   file.puts("DEF-NAME,INST-NAME,LAYER,MATERIAL,AREA,ID,X,Y,Z\n\n") 
   ### title ### add what you want to list here
   ilist.each{|c|
     file.puts(c[0]+","+c[1]+","+c[2]+","+c[3]+","+c[4]+","+c[5]+","+c[6]+","+c[7]+","+c[8])
   }
   file.close
### say it's done...
   UI.messagebox("Component+ Report written into:\n\n"+ccsv+"\n\nInstance+ Report written into:\n\n"+icsv+"\n\n")
### commit undo...
   model.commit_operation
 end#def components
end#class
### Sketchup.send_action "showRubyPanel:"###
### do menu
#if not file_loaded?("ComponentReporter+.rb")
 #   add_separator_to_menu("Plugins")
 #   UI.menu("Plugins").add_item("Component+ Report"){Reporter.components_plus}
#end#if
#file_loaded("ComponentReporter+.rb")
###
###################################################################################################
####   
####                                                              
####
###################################################################################################
#------------------------------------------------ ADD CONTROL POINTS
class AddCpointTool 
def initialize
  @ip = Sketchup::InputPoint.new                   # Define the variable. 
  end
def reset
  @ip.clear
  Sketchup::set_status_text "Left Click to Add Construction Points"   # Status Bar hint. 
  end
def activate
  self.reset
  end
def onMouseMove(flags, x, y, view)
  @ip.pick(view,x,y)                    # Get current Screen coordinates.
  view.tooltip = @ip.tooltip            # Set a tooltip if hovering over geometry.
  end
def onLButtonDown(flags, x, y, view)
  @ip.pick(view,x,y)                       # Get current screen coordinates.
  pt = Sketchup.active_model.active_entities.add_cpoint(@ip.position)   # Add a construction point 
  end
end # class AddCpointTool 
###################################################################################################
####   
####                                                              
####
###################################################################################################
#--------------------------------------------------------------- 3 POIT GRID - EDITABLE
class Grid < Parametric
def create_entities(data, container)
    dx = data["dx"].to_l
    dy = data["dy"].to_l
    nx = data["nx"].to_i
    ny = data["ny"].to_i
    
    # create the vertical lines
    pt = Geom::Point3d.new ORIGIN
    pt2 = pt + [0, dy*ny, 0]
    offset = [dx, 0, 0]
    for i in 0..nx do
        container.add_cline pt, pt2, "..."
        pt += offset
        pt2 += offset
    end
    
    # create the horizontal lines
    pt.set!(ORIGIN)
    pt2 = pt + [dx*nx, 0, 0]
    offset = [0, dy, 0]
    for i in 0..ny do
        container.add_cline pt, pt2, "..."
        pt += offset
        pt2 += offset
    end
    
end
def default_parameters
    defaults = {"dx", 8.feet, "dy", 8.feet, "nx", 4, "ny", 4}
    defaults
end
end
#=============================================================================
class GridTool
@@dx = 1.feet
@@dy = 1.feet
def initialize
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    reset
end
def reset
    @pts = []
    @state = 0
    @ip1.clear
    @drawn = false
    Sketchup::set_status_text "Click for start point"
    @shift_down_time = Time.now
    @nx = 0
    @ny = 0
end
def activate
    self.reset
    @drawn = false
    Sketchup::set_status_text "Spacing", SB_VCB_LABEL
    Sketchup::set_status_text "#{@@dx}", SB_VCB_VALUE
end
def deactivate(view)
    view.invalidate if @drawn
end
def set_current_point(x, y, view)
    if( !@ip.pick(view, x, y, @ip1) )
        return false
    end
    need_draw = true
    
    # Set the tooltip that will be displayed
    view.tooltip = @ip.tooltip
        
    # Compute points
    case @state
    when 0
        @pts[0] = @ip.position
        @pts[4] = @pts[0]
        need_draw = @ip.display? || @drawn
    when 1
        @pts[1] = @ip.position
    when 2
        pt1 = @ip.position
        pt2 = pt1.project_to_line @pts
        vec = pt1 - pt2
        @pts[2] = @pts[1] + vec
        @pts[3] = @pts[0] + vec
    end
    view.invalidate if need_draw
end
def onMouseMove(flags, x, y, view)
    self.set_current_point(x, y, view)
end
def create_grid
    # check for zero size
    if( @nx > 0 && @ny > 0 )
        data = {}
        data["nx"] = @nx
        data["ny"] = @ny
        data["dx"] = @@dx
        data["dy"] = @@dy
        
        # compute the transformation to move it to the right location
        origin = @pts[0]
        xaxis = @pts[1] - @pts[0]
        yaxis = @pts[3] - @pts[0]
        zaxis = xaxis.cross(yaxis)
        t = Geom::Transformation.axes(origin, xaxis, yaxis, zaxis)
        
        # Create the grid
        grid = Grid.new data, t
        
    end
end
def increment_state
    @state += 1
    case @state
    when 1
        @ip1.copy! @ip
        Sketchup::set_status_text "Click for second point"
    when 2
        @ip1.clear
        @pts[2] = @pts[1]
        @pts[3] = @pts[0]
        Sketchup::set_status_text "Click for third point"
    when 3
        self.create_grid
    end
end
def onLButtonDown(flags, x, y, view)
    self.set_current_point(x, y, view)
    self.increment_state
    view.lock_inference
end
def onLButtonUp(flags, x, y, view)
    Sketchup.active_model.select_tool nil if @state == 3
end
def onCancel(flag, view)
    view.invalidate if @drawn
    self.reset
end
# This is called when the user types a value into the VCB
def onUserText(text, view)
    #TODO: Add parsing for different values fo dx and dy
    # The user can enter values for dx and dy
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    
    if( value > 0.0 )
        @@dx = value
        @@dy = value
        view.invalidate
    end
end
def getExtents
    bb = Geom::BoundingBox.new
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? && @ip.display? )
            bb.add @ip.position
        end
    when 1
        bb.add @pts[0]
        bb.add @pts[1]
    when 2
        bb.add @pts
    end
    bb
end
def draw_grid(view)
    # round the length to an even number of grid squares
    xvec = @pts[1] - @pts[0]
    width = xvec.length
    @nx = (width / @@dx).to_i
    width = @nx * @@dx
    xvec.length = width if( width > 0 )
    if( @state > 1 )
        yvec = @pts[3] - @pts[0]
        height = yvec.length
        @ny = (height / @@dy).to_i
        height = @ny * @@dy
        yvec.length = height if( height > 0 )
    else
        @ny = 0
    end
    # Check for special cases where there is not really anything to draw
    if( @ny < 1 )
        return if @nx < 1
        
        # Draw a single line to avoid confusion in the case where
        # we are just starting to drag out the grid
        view.draw(GL_LINE_STRIP, @pts[0], @pts[1])
        
        # Draw tick marks
        offset = xvec.clone
        viewdir = view.camera.direction
        vec = offset.cross viewdir
        return if vec.length == 0.0
        vec.length = view.pixels_to_model 6, @pts[0]
        offset.length = @@dx
        pt1 = @pts[0]
        pt2 = pt1 + vec
        view.line_stipple = 0
        for i in 0..@nx do
            view.draw(GL_LINE_STRIP, pt1, pt2)
            pt1 += offset
            pt2 += offset
        end
        
        return
        
    elsif( @nx < 1 )
        view.draw(GL_LINE_STRIP, @pts[0], @pts[3])
        return
    end
    
    view.drawing_color = "black"
    
    # draw the vertical lines
    pt1 = @pts[0]
    pt2 = pt1 + yvec
    offset = xvec.clone
    offset.length = @@dx
    for i in 0..@nx do
        view.draw(GL_LINE_STRIP, pt1, pt2)
        pt1 += offset
        pt2 += offset
    end
    
    # draw the horizontal lines
    pt1 = @pts[0]
    pt2 = pt1 + xvec
    offset = yvec
    offset.length = @@dy
    for i in 0..@ny do
        view.draw(GL_LINE_STRIP, pt1, pt2)
        pt1 += offset
        pt2 += offset
    end
    
    true
end
def draw(view)
    @drawn = false
    
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end
    # show the rectangle
    view.line_stipple = "..."
    if( @state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line(@ip1, @ip)
        #inference_locked = view.inference_locked?
        #view.line_width = 3 if inference_locked
        @drawn = draw_grid(view)
        #view.line_width = 1 if inference_locked
    elsif( @state > 1 )
        @drawn = draw_grid(view)
    end
    view.line_stipple = 0
end
def onKeyDown(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            view.lock_inference
        elsif( @state == 1 )
            view.lock_inference @ip, @ip1
        else
            view.lock_inference @ip
        end
    end
end
def onKeyUp(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end
end # of class GridTool
###################################################################################################
####   
####                                                              
####
###################################################################################################
#--------------------------------------------------------------------------------------  ROTATED RECTANGLE
class RectangleTool
def initialize
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    reset
end
def reset
    @pts = []
    @state = 0
    @ip1.clear
    @drawn = false
    Sketchup::set_status_text "", SB_VCB_LABEL
    Sketchup::set_status_text "", SB_VCB_VALUE
    Sketchup::set_status_text "Click for start point"
    @shift_down_time = Time.now
end
def activate
    self.reset
end
def deactivate(view)
    view.invalidate if @drawn
end
def set_current_point(x, y, view)
    if( !@ip.pick(view, x, y, @ip1) )
        return false
    end
    need_draw = true
    
    # Set the tooltip that will be displayed
    view.tooltip = @ip.tooltip
        
    # Compute points
    case @state
    when 0
        @pts[0] = @ip.position
        @pts[4] = @pts[0]
        need_draw = @ip.display? || @drawn
    when 1
        @pts[1] = @ip.position
        @width = @pts[0].distance @pts[1]
        Sketchup::set_status_text @width.to_s, SB_VCB_VALUE
    when 2
        pt1 = @ip.position
        pt2 = pt1.project_to_line @pts
        vec = pt1 - pt2
        @height = vec.length
        if( @height > 0 )
            # test for a square
            square_point = pt2.offset(vec, @width)
            if( view.pick_helper.test_point(square_point, x, y) )
                @height = @width
                @pts[2] = @pts[1].offset(vec, @height)
                @pts[3] = @pts[0].offset(vec, @height)
                view.tooltip = "Square"
            else
                @pts[2] = @pts[1].offset(vec)
                @pts[3] = @pts[0].offset(vec)
            end
        else
            @pts[2] = @pts[1]
            @pts[3] = @pts[0]
        end
        Sketchup::set_status_text @height.to_s, SB_VCB_VALUE
    end
    view.invalidate if need_draw
end
def onMouseMove(flags, x, y, view)
    self.set_current_point(x, y, view)
end
def create_rectangle
    # check for zero height
    if( @pts[0] != @pts[3] )
        Sketchup.active_model.active_entities.add_face @pts
    end
    self.reset
end
def increment_state
    @state += 1
    case @state
    when 1
        @ip1.copy! @ip
        Sketchup::set_status_text "Click for second point"
        Sketchup::set_status_text "Width", SB_VCB_LABEL
        Sketchup::set_status_text "", SB_VCB_VALUE
    when 2
        @ip1.clear
        Sketchup::set_status_text "Click for third point"
        Sketchup::set_status_text "Height", SB_VCB_LABEL
        Sketchup::set_status_text "", SB_VCB_VALUE
    when 3
        self.create_rectangle
    end
end
def onLButtonDown(flags, x, y, view)
    self.set_current_point(x, y, view)
    self.increment_state
    view.lock_inference
end
def onCancel(flag, view)
    view.invalidate if @drawn
    self.reset
end
# This is called when the user types a value into the VCB
def onUserText(text, view)
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    
    case @state
    when 1
        # update the width
        vec = @pts[1] - @pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            @pts[1] = @pts[0].offset(vec)
            view.invalidate
            self.increment_state
        end
    when 2
        # update the height
        vec = @pts[3] - @pts[0]
        if( vec.length > 0.0 )
            vec.length = value
            @pts[2] = @pts[1].offset(vec)
            @pts[3] = @pts[0].offset(vec)
            self.increment_state
        end
    end
end
def getExtents
    bb = Geom::BoundingBox.new
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? && @ip.display? )
            bb.add @ip.position
        end
    when 1
        bb.add @pts[0]
        bb.add @pts[1]
    when 2
        bb.add @pts
    end
    bb
end
def draw(view)
    @drawn = false
    
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end
    # show the rectangle
    if( @state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line(@ip1, @ip)
        inference_locked = view.inference_locked?
        view.line_width = 3 if inference_locked
        view.draw(GL_LINE_STRIP, @pts[0], @pts[1])
        view.line_width = 1 if inference_locked
        @drawn = true
    elsif( @state > 1 )
        # draw the curve
        view.drawing_color = "black"
        view.draw(GL_LINE_STRIP, @pts)
        @drawn = true
    end
end
def onKeyDown(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && rpt == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            view.lock_inference
        elsif( @state == 0 )
            view.lock_inference @ip
        elsif( @state == 1 )
            view.lock_inference @ip, @ip1
        end
    end
end
def onKeyUp(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end
end # of class RectangleTool
###################################################################################################
####   
####                       fun shapes                                       
####
###################################################################################################

def cos_hip(x)
  return (Math::exp(x) + Math::exp(-x))/2
end
def superellipse_superellipsoid
  prompt = ["# Iterations : ", "x Scale Factor : ", "y Scale Factor : ", "z Scale Factor : ", "z Squareness : ", "x-y Squareness : "]
  params = [5, 1.0, 1.0, 1.0, 1.0, 3.0]
  result = inputbox(prompt, params, "Imput Superellipse/Superellipsoid Parameters")
  if result
    it = [2, result[0]].max
    rx = [1.0, result[1].abs].max
    ry = [1.0, result[2].abs].max
    rz = [1.0, result[3].abs].max
    n1 = [0.05, result[4].abs].max
    n2 = [0.05, result[5].abs].max
  else
    return
  end
  entities = Sketchup.active_model.entities
  group = entities.add_group
  group_ents = group.entities
  mesh_ss1 = Geom::PolygonMesh.new
  mesh_ss2 = Geom::PolygonMesh.new
  mesh_ss3 = Geom::PolygonMesh.new
  mesh_ss4 = Geom::PolygonMesh.new
  mesh_ss5 = Geom::PolygonMesh.new
  mesh_ss6 = Geom::PolygonMesh.new
  mesh_ss7 = Geom::PolygonMesh.new
  mesh_ss8 = Geom::PolygonMesh.new
  indexes = [0]
  aux_vect = []
  for ind_u in (0..2*it)
    u = (Math::PI/2)*ind_u/(2*it)
    for ind_v in (0..2*it)
      v = (Math::PI/2)*ind_v/(2*it)
      x = rx*(Math::cos(u)**n1)*(Math::cos(v)**n2)
      y = ry*(Math::cos(u)**n1)*(Math::sin(v)**n2)
      z = rz*(Math::sin(u)**n1)
      pt = Geom::Point3d.new(100*x, 100*y, 100*z)
      index = mesh_ss1.add_point(pt)
      pt = Geom::Point3d.new(-100*x, -100*y, 100*z)
      index = mesh_ss2.add_point(pt)
      pt = Geom::Point3d.new(-100*x, 100*y, -100*z)
      index = mesh_ss3.add_point(pt)
      pt = Geom::Point3d.new(100*x, -100*y, -100*z)
      index = mesh_ss4.add_point(pt)
      pt = Geom::Point3d.new(100*x, 100*y, -100*z)
      index = mesh_ss5.add_point(pt)
      pt = Geom::Point3d.new(-100*x, -100*y, -100*z)
      index = mesh_ss6.add_point(pt)
      pt = Geom::Point3d.new(-100*x, 100*y, 100*z)
      index = mesh_ss7.add_point(pt)
      pt = Geom::Point3d.new(100*x, -100*y, 100*z)
      index = mesh_ss8.add_point(pt)
      indexes.push(index)
    end
  end
  for ind_u in (1..2*it)
    for ind_v in (1..2*it)
      aux_ind = (2*it+1)*(ind_u-1) + ind_v
      aux_vect = [indexes[aux_ind], indexes[aux_ind+1], indexes[aux_ind+1+2*it]]
      mesh_ss1.add_polygon(aux_vect)
      mesh_ss2.add_polygon(aux_vect)
      mesh_ss3.add_polygon(aux_vect)
      mesh_ss4.add_polygon(aux_vect)
      aux_vect = [indexes[aux_ind], indexes[aux_ind+1+2*it], indexes[aux_ind+1]]
      mesh_ss5.add_polygon(aux_vect)
      mesh_ss6.add_polygon(aux_vect)
      mesh_ss7.add_polygon(aux_vect)
      mesh_ss8.add_polygon(aux_vect)
      aux_vect = [indexes[aux_ind+1], indexes[aux_ind+2+2*it], indexes[aux_ind+1+2*it]]
      mesh_ss1.add_polygon(aux_vect)
      mesh_ss2.add_polygon(aux_vect)
      mesh_ss3.add_polygon(aux_vect)
      mesh_ss4.add_polygon(aux_vect)
      aux_vect = [indexes[aux_ind+1], indexes[aux_ind+1+2*it], indexes[aux_ind+2+2*it]]
      mesh_ss5.add_polygon(aux_vect)
      mesh_ss6.add_polygon(aux_vect)
      mesh_ss7.add_polygon(aux_vect)
      mesh_ss8.add_polygon(aux_vect)
    end
  end
  group_ents.add_faces_from_mesh(mesh_ss1)
  group_ents.add_faces_from_mesh(mesh_ss2)
  group_ents.add_faces_from_mesh(mesh_ss3)
  group_ents.add_faces_from_mesh(mesh_ss4)
  group_ents.add_faces_from_mesh(mesh_ss5)
  group_ents.add_faces_from_mesh(mesh_ss6)
  group_ents.add_faces_from_mesh(mesh_ss7)
  group_ents.add_faces_from_mesh(mesh_ss8)
end
def tear_drop
  prompt = ["# Iterations : "]
  params = [5]
  result = inputbox(prompt, params, "Imput Tear Drop Parameters")
  if result
    it = [2, result[0]].max
  else
    return
  end
  entities = Sketchup.active_model.entities
  group = entities.add_group
  group_ents = group.entities
  mesh_td = Geom::PolygonMesh.new
  indexes = [0]
  aux_vect = []
  for ind_u in (0..2*it)
    u = Math::PI*ind_u/(2*it)
    for ind_v in (0..2*it-1)
      v = 2*Math::PI*ind_v/(2*it)
      x = 0.5*(1 - Math::cos(u))*Math::sin(u)*Math::cos(v)
      y = 0.5*(1 - Math::cos(u))*Math::sin(u)*Math::sin(v)
      z = Math::cos(u)
      pt = Geom::Point3d.new(100*x, 100*y, 100*z)
      index = mesh_td.add_point(pt)
      indexes.push(index)
    end
  end
  for ind_u in (1..2*it)
    for ind_v in (1..2*it-1)
      aux_ind = 2*it*(ind_u-1) + ind_v
      aux_vect = [indexes[aux_ind], indexes[aux_ind+2*it], indexes[aux_ind+1]]
      mesh_td.add_polygon(aux_vect)
      aux_vect = [indexes[aux_ind+1], indexes[aux_ind+2*it], indexes[aux_ind+1+2*it]]
      mesh_td.add_polygon(aux_vect)
    end
    aux_vect = [indexes[aux_ind+1], indexes[aux_ind+1+2*it], indexes[aux_ind+2-2*it]]
    mesh_td.add_polygon(aux_vect)
    aux_vect = [indexes[aux_ind+2-2*it], indexes[aux_ind+1+2*it], indexes[aux_ind+2]]
    mesh_td.add_polygon(aux_vect)
  end
  group_ents.add_faces_from_mesh(mesh_td)
end
def horn_shape
  prompt = ["# Iterations : "]
  params = [5]
  result = inputbox(prompt, params, "Imput Horn Shape Parameters")
  if result
    it = [2, result[0]].max
  else
    return
  end
  entities = Sketchup.active_model.entities
  group = entities.add_group
  group_ents = group.entities
  mesh_hs = Geom::PolygonMesh.new
  indexes = [0]
  aux_vect = []
  for ind_u in (0..2*it)
    u = ind_u/(2.0*it)
    for ind_v in (0..2*it-1)
      v = 2*Math::PI*ind_v/(2*it)
      x = (2 + u*Math::cos(v))*Math::sin(2*Math::PI*u)
      y = (2 + u*Math::cos(v))*Math::cos(2*Math::PI*u) + 2*u
      z = u*Math::sin(v)
      pt = Geom::Point3d.new(100*x, 100*y, 100*z)
      index = mesh_hs.add_point(pt)
      indexes.push(index)
    end
  end
  for ind_u in (1..2*it)
    for ind_v in (1..2*it-1)
      aux_ind = 2*it*(ind_u-1) + ind_v
      aux_vect = [indexes[aux_ind], indexes[aux_ind+1], indexes[aux_ind+2*it]]
      mesh_hs.add_polygon(aux_vect)
      aux_vect = [indexes[aux_ind+1], indexes[aux_ind+1+2*it], indexes[aux_ind+2*it]]
      mesh_hs.add_polygon(aux_vect)
    end
    aux_vect = [indexes[aux_ind+1], indexes[aux_ind+2-2*it], indexes[aux_ind+1+2*it]]
    mesh_hs.add_polygon(aux_vect)
    aux_vect = [indexes[aux_ind+2-2*it], indexes[aux_ind+2], indexes[aux_ind+1+2*it]]
    mesh_hs.add_polygon(aux_vect)
  end
  group_ents.add_faces_from_mesh(mesh_hs)
end
def misc_torus
  prompt = ["# Iterations : ", "c Value : "]
  params = [5, 1.5]
  result = inputbox(prompt, params, "Imput Misc Torus Parameters")
  if result
    it = [2, result[0]].max
    c = result[1].abs
  else
    return
  end
  entities = Sketchup.active_model.entities
  group = entities.add_group
  group_ents = group.entities
  mesh_mt = Geom::PolygonMesh.new
  indexes = [0]
  aux_vect = []
  for ind_u in (-it..it-1)
    u = Math::PI*ind_u/it
    for ind_v in (-it..it-1)
      v = Math::PI*ind_v/it
      x = Math::cos(u)*(c + Math::sin(v)*Math::cos(u) - Math::sin(2*v)*Math::sin(u)/2)
      y = Math::sin(u)*(c + Math::sin(v)*Math::cos(u) - Math::sin(2*v)*Math::sin(u)/2)
      z = Math::sin(u)*Math::sin(v) + Math::cos(u)*Math::sin(2*v)/2
      pt = Geom::Point3d.new(100*x, 100*y, 100*z)
      index = mesh_mt.add_point(pt)
      indexes.push(index)
    end
  end
  for ind_u in (1..2*it-1)
    for ind_v in (1..2*it-1)
      aux_ind = 2*it*(ind_u-1) + ind_v
      aux_vect = [indexes[aux_ind], indexes[aux_ind+2*it], indexes[aux_ind+1]]
      mesh_mt.add_polygon(aux_vect)
      aux_vect = [indexes[aux_ind+1], indexes[aux_ind+2*it], indexes[aux_ind+1+2*it]]
      mesh_mt.add_polygon(aux_vect)
    end
    aux_vect = [indexes[aux_ind+1], indexes[aux_ind+1+2*it], indexes[aux_ind+2-2*it]]
    mesh_mt.add_polygon(aux_vect)
    aux_vect = [indexes[aux_ind+2-2*it], indexes[aux_ind+1+2*it], indexes[aux_ind+2]]
    mesh_mt.add_polygon(aux_vect)
  end
  for ind_v in (1..2*it-1)
    aux_vect = [indexes[ind_v], indexes[ind_v+1], indexes[2*it*(2*it-1)+ind_v+1]]
    mesh_mt.add_polygon(aux_vect)
    aux_vect = [indexes[ind_v], indexes[2*it*(2*it-1)+ind_v+1], indexes[2*it*(2*it-1)+ind_v]]
    mesh_mt.add_polygon(aux_vect)
  end
  aux_vect = [indexes[2*it*(2*it-1)+1], indexes[4*it*it], indexes[2*it]]
  mesh_mt.add_polygon(aux_vect)
  aux_vect = [indexes[1], indexes[2*it*(2*it-1)+1], indexes[2*it]]
  mesh_mt.add_polygon(aux_vect)
  group_ents.add_faces_from_mesh(mesh_mt)
end
def jet_surface
  prompt = ["# Iterations : "]
  params = [5]
  result = inputbox(prompt, params, "Imput Jet Surface Parameters")
  if result
    it = [2, result[0]].max
  else
    return
  end
  entities = Sketchup.active_model.entities
  group = entities.add_group
  group_ents = group.entities
  mesh_js = Geom::PolygonMesh.new
  indexes = [0]
  aux_vect = []
  for ind_u in (0..2*it)
    u = Math::PI*ind_u/(2*it)
    for ind_v in (0..2*it-1)
      v = 2*Math::PI*ind_v/(2*it)
      x = (1 - cos_hip(u))*Math::sin(u)*Math::cos(v)/2
      y = (1 - cos_hip(u))*Math::sin(u)*Math::sin(v)/2
      z = cos_hip(u)
      pt = Geom::Point3d.new(100*x, 100*y, 100*z)
      index = mesh_js.add_point(pt)
      indexes.push(index)
    end
  end
  for ind_u in (1..2*it)
    for ind_v in (1..2*it-1)
      aux_ind = 2*it*(ind_u-1) + ind_v
      aux_vect = [indexes[aux_ind], indexes[aux_ind+1], indexes[aux_ind+2*it]]
      mesh_js.add_polygon(aux_vect)
      aux_vect = [indexes[aux_ind+1], indexes[aux_ind+1+2*it], indexes[aux_ind+2*it]]
      mesh_js.add_polygon(aux_vect)
    end
    aux_vect = [indexes[aux_ind+1], indexes[aux_ind+2-2*it], indexes[aux_ind+1+2*it]]
    mesh_js.add_polygon(aux_vect)
    aux_vect = [indexes[aux_ind+2-2*it], indexes[aux_ind+2], indexes[aux_ind+1+2*it]]
    mesh_js.add_polygon(aux_vect)
  end
  group_ents.add_faces_from_mesh(mesh_js)
end
def pisot_triaxial
  prompt = ["# Iterations : "]
  params = [5]
  result = inputbox(prompt, params, "Imput Pisot Triaxial Parameters")
  if result
    it = [2, result[0]].max
  else
    return
  end
  entities = Sketchup.active_model.entities
  group = entities.add_group
  group_ents = group.entities
  mesh_pt = Geom::PolygonMesh.new
  indexes = [0]
  aux_vect = []
  for ind_u in (0..2*it-1)
    u = 2*Math::PI*ind_u/(2*it)
    for ind_v in (0..2*it-1)
      v = 2*Math::PI*ind_v/(2*it)
      x = 0.655866*Math::cos(1.03002+u)*(2 + Math::cos(v))
      y = 0.754878*Math::cos(1.40772-u)*(2 + 0.868837*Math::cos(2.43773+v))
      z = 0.868837*Math::cos(2.43773+u)*(2 + 0.495098*Math::cos(0.377696-v))
      pt = Geom::Point3d.new(100*x, 100*y, 100*z)
      index = mesh_pt.add_point(pt)
      indexes.push(index)
    end
  end
  for ind_u in (1..2*it-1)
    for ind_v in (1..2*it-1)
      aux_ind = 2*it*(ind_u-1) + ind_v
      aux_vect = [indexes[aux_ind], indexes[aux_ind+1], indexes[aux_ind+2*it]]
      mesh_pt.add_polygon(aux_vect)
      aux_vect = [indexes[aux_ind+1], indexes[aux_ind+1+2*it], indexes[aux_ind+2*it]]
      mesh_pt.add_polygon(aux_vect)
    end
    aux_vect = [indexes[aux_ind+1], indexes[aux_ind+2-2*it], indexes[aux_ind+1+2*it]]
    mesh_pt.add_polygon(aux_vect)
    aux_vect = [indexes[aux_ind+2-2*it], indexes[aux_ind+2], indexes[aux_ind+1+2*it]]
    mesh_pt.add_polygon(aux_vect)
  end
  for ind_v in (1..2*it-1)
    aux_vect = [indexes[ind_v], indexes[2*it*(2*it-1)+ind_v+1], indexes[ind_v+1]]
    mesh_pt.add_polygon(aux_vect)
    aux_vect = [indexes[ind_v], indexes[2*it*(2*it-1)+ind_v], indexes[2*it*(2*it-1)+ind_v+1]]
    mesh_pt.add_polygon(aux_vect)
  end
  aux_vect = [indexes[2*it*(2*it-1)+1], indexes[2*it], indexes[4*it*it]]
  mesh_pt.add_polygon(aux_vect)
  aux_vect = [indexes[1], indexes[2*it], indexes[2*it*(2*it-1)+1]]
  mesh_pt.add_polygon(aux_vect)
  group_ents.add_faces_from_mesh(mesh_pt)
end
def elliptic_torus
  prompt = ["# Iterations : ", "c Value : "]
  params = [5, 1.2]
  result = inputbox(prompt, params, "Imput Elliptic Torus Parameters")
  if result
    it = [2, result[0]].max
    c = result[1].abs
  else
    return
  end
  entities = Sketchup.active_model.entities
  group = entities.add_group
  group_ents = group.entities
  mesh_et = Geom::PolygonMesh.new
  indexes = [0]
  aux_vect = []
  for ind_u in (-it..it-1)
    u = Math::PI*ind_u/it
    for ind_v in (-it..it-1)
      v = Math::PI*ind_v/it
      x = (c + Math::cos(v))*Math::cos(u)
      y = (c + Math::cos(v))*Math::sin(u)
      z = Math::sin(v) + Math::cos(v)
      pt = Geom::Point3d.new(100*x, 100*y, 100*z)
      index = mesh_et.add_point(pt)
      indexes.push(index)
    end
  end
  for ind_u in (1..2*it-1)
    for ind_v in (1..2*it-1)
      aux_ind = 2*it*(ind_u-1) + ind_v
      aux_vect = [indexes[aux_ind], indexes[aux_ind+2*it], indexes[aux_ind+1]]
      mesh_et.add_polygon(aux_vect)
      aux_vect = [indexes[aux_ind+1], indexes[aux_ind+2*it], indexes[aux_ind+1+2*it]]
      mesh_et.add_polygon(aux_vect)
    end
    aux_vect = [indexes[aux_ind+1], indexes[aux_ind+1+2*it], indexes[aux_ind+2-2*it]]
    mesh_et.add_polygon(aux_vect)
    aux_vect = [indexes[aux_ind+2-2*it], indexes[aux_ind+1+2*it], indexes[aux_ind+2]]
    mesh_et.add_polygon(aux_vect)
  end
  for ind_v in (1..2*it-1)
    aux_vect = [indexes[ind_v], indexes[ind_v+1], indexes[2*it*(2*it-1)+ind_v+1]]
    mesh_et.add_polygon(aux_vect)
    aux_vect = [indexes[ind_v], indexes[2*it*(2*it-1)+ind_v+1], indexes[2*it*(2*it-1)+ind_v]]
    mesh_et.add_polygon(aux_vect)
  end
  aux_vect = [indexes[2*it*(2*it-1)+1], indexes[4*it*it], indexes[2*it]]
  mesh_et.add_polygon(aux_vect)
  aux_vect = [indexes[1], indexes[2*it*(2*it-1)+1], indexes[2*it]]
  mesh_et.add_polygon(aux_vect)
  group_ents.add_faces_from_mesh(mesh_et)
end
def limpet_torus
  prompt = ["# Iterations : "]
  params = [5]
  result = inputbox(prompt, params, "Imput Limpet Torus Parameters")
  if result
    it = [2, result[0]].max
  else
    return
  end
  entities = Sketchup.active_model.entities
  group = entities.add_group
  group_ents = group.entities
  mesh_lt = Geom::PolygonMesh.new
  indexes = [0]
  aux_vect = []
  for ind_u in (-it..it-1)
    u = Math::PI*ind_u/it
    for ind_v in (-it..it-1)
      v = Math::PI*ind_v/it
      x = Math::cos(u)/(Math::sqrt(2) + Math::sin(v))
      y = Math::sin(u)/(Math::sqrt(2) + Math::sin(v))
      z = 1/(Math::sqrt(2) + Math::cos(v))
      pt = Geom::Point3d.new(100*x, 100*y, 100*z)
      index = mesh_lt.add_point(pt)
      indexes.push(index)
    end
  end
  for ind_u in (1..2*it-1)
    for ind_v in (1..2*it-1)
      aux_ind = 2*it*(ind_u-1) + ind_v
      aux_vect = [indexes[aux_ind], indexes[aux_ind+1], indexes[aux_ind+2*it]]
      mesh_lt.add_polygon(aux_vect)
      aux_vect = [indexes[aux_ind+1], indexes[aux_ind+1+2*it], indexes[aux_ind+2*it]]
      mesh_lt.add_polygon(aux_vect)
    end
    aux_vect = [indexes[aux_ind+1], indexes[aux_ind+2-2*it], indexes[aux_ind+1+2*it]]
    mesh_lt.add_polygon(aux_vect)
    aux_vect = [indexes[aux_ind+2-2*it], indexes[aux_ind+2], indexes[aux_ind+1+2*it]]
    mesh_lt.add_polygon(aux_vect)
  end
  for ind_v in (1..2*it-1)
    aux_vect = [indexes[ind_v], indexes[2*it*(2*it-1)+ind_v+1], indexes[ind_v+1]]
    mesh_lt.add_polygon(aux_vect)
    aux_vect = [indexes[ind_v], indexes[2*it*(2*it-1)+ind_v], indexes[2*it*(2*it-1)+ind_v+1]]
    mesh_lt.add_polygon(aux_vect)
  end
  aux_vect = [indexes[2*it*(2*it-1)+1], indexes[2*it], indexes[4*it*it]]
  mesh_lt.add_polygon(aux_vect)
  aux_vect = [indexes[1], indexes[2*it], indexes[2*it*(2*it-1)+1]]
  mesh_lt.add_polygon(aux_vect)
  group_ents.add_faces_from_mesh(mesh_lt)
end


###################################################################################################
####   
####                                               POLAR LINE               
####
###################################################################################################


class PolarLine

def initialize
	@ip = nil
    @ip1 = nil
	lg = nil
	ang = nil
	pl = nil
end

def reset(view)
	return @ip1.position
    @ip1.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
end

def activate
	@ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    Sketchup::set_status_text("Specify first point : ", SB_PROMPT)
    self.reset(nil)
end

def deactivate(view)
    view.invalidate if @drawn
end

def onCancel(flag, view)
    self.reset(view)
end

def onMouseMove(flags, x, y, view)
	@ip.pick view, x, y
	if( @ip != @ip1 )
		view.invalidate if( @ip.display? or @ip1.display? )
		@ip1.copy! @ip
		view.tooltip = @ip1.tooltip
	end
end

def onLButtonDown(flags, x, y, view)
	@ip1.pick view, x, y
	if( @ip1.valid? )
		self.create_geometry(@ip1.position,view)
		self.reset(view)
	end
	view.lock_inference
end

def draw(view)
	self.draw_geometry(@ip1.position, view) if (@ip1.valid?) and (@ip1.display?)
end

def create_geometry(p1, view)
    view.model.entities.add_cpoint(p1)
	ents = Sketchup.active_model.active_entities
	n = ents.count - 1
	Sketchup.send_action "selectSelectionTool:"
	lst =["","","Red-Green|Red-Blue|Green-Blue"]
	prompt = ["Length: ","trigo angle: ","3D plan: "]
	values = [$lg,$ang,$pl]
	result = inputbox(prompt,values,lst,"Polar parameters")
	if result
		$lg = result[0];$ang = result[1];$pl = result[2]
		puts("First point: " + p1.to_s + "\nLength: " + result[0].to_s + "\nAngle: " + result[1].to_s + "\nPlan: " + result[2])
		case Sketchup.active_model.options[0][2]
			when 0;lg = result[0].to_f
			when 1;lg = result[0].to_f.feet
			when 2;lg = result[0].to_f.mm
			when 3;lg = result[0].to_f.cm
			when 4;lg = result[0].to_f.m
		end
		ang = result[1].to_f.degrees
		pl = result[2]
		case pl
			when "Red-Green"
			x = p1[0] + lg * Math.cos(ang)
			y = p1[1] + lg * Math.sin(ang)
			z = p1[2]
			when "Red-Blue"
			x = p1[0] + lg * Math.cos(ang)
			y = p1[1]
			z = p1[2] + lg * Math.sin(ang)
			when "Green-Blue"
			x = p1[0]
			y = p1[1] + lg * Math.cos(ang)
			z = p1[2] + lg * Math.sin(ang)
		end
		p2 = [x,y,z]
		ents[n].erase!
		ents.add_edges(p1,p2) if p1 and p2
		puts "\nPolarLine Ok."
	else
		ents[n].erase!
		return nil
	end
end

def draw_geometry(p1, view)
    view.draw_points(p1)
end


end

def polarline
    Sketchup.active_model.select_tool PolarLine.new
end

###################################################################################################

